#!/usr/bin/perl -w

local %config;

##############################################
#	Network interfaces presented on the host
##############################################
local @interfaces;

##########################################################
#	Domain names which are affected by IP address change
##########################################################
local @domains;

################################################################
#	int main(int argc, char** argv, char** envp) { return 0; }
################################################################

#################################
#	By default just print usage
#################################
if (-1 == $#ARGV || ( (0 == $#ARGV) && ('-h' eq $ARGV[0]) ) ) {
	&print_usage($0);
	exit 0;
}

$config{MAP_FILE} = $ARGV[0];

&read_config();

##################
#	TODO: Add here check for zero-sized file
##################
if (-e $config{MAP_FILE}) {

	########################################
	#	Use map_file and reconfigure Plesk
	########################################
	if (&process_map_file($config{MAP_FILE}) != 0) {

		printf("IP addresses weren't mapped\n");

	} else {

		printf("Mapping procedure successfully done\n");
	}

} else {

	##############################
	#	Create template map file
	##############################
	if (&create_map_file($config{MAP_FILE}) != 0) {
		
		########################
		#	Map wasn't created
		########################
		printf("Map file wasn't created\n");
		exit 1;

	} else {

		###################################
		#	Map file successfully created
		###################################
		printf("IP map file template %s was successfully created\n", 
			$config{MAP_FILE});
		printf("Edit it manually to reflect your configuration\n");
		exit 0;

	}

	exit 0;
}

exit 0;

########################################

exit 1;

######################################################################
#	Read psa.conf and .psa.shadow and extract neccessary information
######################################################################
sub read_config()
{

	open FCONF, "< /etc/psa/psa.conf" 
		or die "Can't open Plesk configurational file\n";

	while (<FCONF>) {
		s/\#.*$//;
		m/^\s*(\w+)\s+(.+?)\s*$/;
		next unless $1;
		$config{$1} = $2;
	}

	close FCONF;

	$config{LOGIN} = 'admin';
	$config{DBNAME} = 'psa';

	###################################
	#	Read administrator's password
	###################################
	open PASSWD, "< /etc/psa/.psa.shadow"
		or die "Can't get Plesk administrator's password\n";
	$config{PASSWORD} = <PASSWD>;
	chomp $config{PASSWORD};
	close PASSWD;
	
	$config{MYSQL} = $config{MYSQL_BIN_D}.'/mysql -s -N -u'.shellArgQuote($config{LOGIN}).' -p'.shellArgQuote($config{PASSWORD}).' -D'.$config{DBNAME};

	if (-e '/etc/SuSE-release' or -e '/etc/mandrake-release' or -e '/etc/debian_version') {
		$config{MYSQL_SCRIPT} = $config{PRODUCT_RC_D} . '/mysql';
	} else {
		$config{MYSQL_SCRIPT} = $config{PRODUCT_RC_D} . '/mysqld';
	}

	$config{IFMNG}     = $config{PRODUCT_ROOT_D}.'/admin/bin/ifmng';
	$config{DNSMNG}    = $config{PRODUCT_ROOT_D}.'/admin/bin/dnsmng';
	$config{MYAPCIRST} = $config{PRODUCT_ROOT_D}.'/admin/bin/websrvmng';

	return 0;
}

##############
#	Usage...
##############
sub print_usage()
{

	my ($progname) = @_;

	my $usage = <<USAGE_INFO;
 Plesk reconfigurator - utility to change IP addresses used by 
 Plesk Server Administrator

 usage:
	$progname <map_file>

 If <map_file> doesn't exists - template will be created, otherwise
 it will be used to map IP addresses.
USAGE_INFO

	printf($usage);
}

##########################################################
#	Create template of map file for IP addresses mapping
#	User friendly - prints state messages
##########################################################
sub create_map_file() {

	my ($filename) = @_;

	if (-e $filename) {

		###################################################################
		#	Actually, I can ask interactive here - wish user rewrite file
		#	or not? I'll do this if I have time
		###################################################################

		printf("File %s already exists!\n", $filename);
		return -1;
	}

	##################################
	#	Ensure that MySQL is running
	##################################
	if (&check_mysql() != 0) {

		############################################
		#	If can't connect - attempt to start it
		############################################
		printf("Attempting to start mysql service\n");
		system($config{MYSQL_SCRIPT} . " start");
		sleep(3);

		if (&check_mysql() != 0) {
			printf("Can't connect to Plesk database\n");
			return -1;
		}
	}

	open(MAPFILE, "> $filename");

	#################################################
	#	Add comment to the file, describing what to 
	#	do with it's contents
	#################################################
	my $usage = <<USAGE;
# You should edit IP addresses, netmasks and interfaces to reflect your
# future settings. If you don't want the IP to be changed - leave it untouched,
# comment out it's line or remove entire line from the file.

USAGE

	print MAPFILE $usage;

	####################################################
	#	Retrieve all IP addresses, registered in Plesk
	#	dump them into the file
	####################################################
	my $query = "SELECT iface,ip_address,mask FROM IP_Addresses";
	my $command = $config{MYSQL} . " -e '$query'";

	#########################
	#	Execute MySQL query
	#########################
	open(QUERY, "$command |");
	while (<QUERY>) {
		if (m/([\S]+)\s([\S]+)\s([\S]+)/) {

			my $iface   = $1;
			my $address = $2;
			my $mask    = $3;

			print MAPFILE "$iface:$address $mask -> $iface:$address $mask\n";
		}
	}
	close(QUERY);

	close(MAPFILE);

	return 0;
}

################################################################
#	Check that MySQL settings are correct and MySQL is running
#	Using vanilla "system" instead of DBI functionality to 
#	reduce amount of modules required for reconfigurator
#	operations. We're not concerning speed.
################################################################
sub check_mysql() {

	printf("Attempting to connect to MySQL: "); 

	my $res = system($config{MYSQL} . " -e '' 2> /dev/null");

	printf("%s\n", ($res ? "failed" : "ok"));

	return $res;
}

######################################################
#	Proceeds map file and reconfigure Plesk database
#	and system configurational files
######################################################
sub process_map_file() {

	my ($filename) = @_;

	#####################################################
	#	Associative array of IP addresses, ready to map
	#	old_ip -> (new_iface, new_ip, new_mask)
	#####################################################
	my %ips;

	#####################################################
	#	List of IP addresses, which shouldn't be mapped
	#####################################################
	my @skipped;

	######################################################
	#	Flag to mark if errors occured during IP mapping
	######################################################
	my $were_errors = 0;

	#######################################
	#	Current line number in input file
	#######################################
	my $line = 0;

	#####################################
	#	Add a check for zero-sized file
	#####################################
	if (! -e $filename) {
		return -1;
	}

	printf("Using mapfile %s - start checking\n", $filename);

	open(MAPFILE, "< $filename");

	###################
	#	Parse mapfile
	###################
LINE:
	while (<MAPFILE>) {

		$line++;

		####################################
		#	Skip commented and empty lines
		####################################
		if (m/^\s*(#.*)?$/) { next; }

		###################
		#	Analize it...
		###################
		my $spc = '\s';
		my $iface = '[\w]+';
		my $ip = '\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}';
		my $mask = '\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}';
		my $comment = '\#\.';

		if (m/^$spc*($iface)$spc*:$spc*($ip)$spc+($mask)$spc*->
			   $spc*($iface)$spc*:$spc*($ip)$spc+($mask)$spc*
			   ($comment)?$/x) {

			##############################################
			#	Watch for all fields being correctly set
			##############################################
			my $old_iface = $1;
			my $old_ip    = $2;
			my $old_mask  = $3;

			my $new_iface = $4;
			my $new_ip    = $5;
			my $new_mask  = $6;

			#####################################################
			#	Check that new interface is presented in system
			#####################################################
			if (&check_iface($new_iface) != 0) {

				###################################
				#	Wrong interface specification
				###################################
				printf("Wrong interface specification on line %s - ".
					"line skipped\n", $line);
				$were_errors = 1;
				next;
			}
			if (&check_ip($old_ip) != 0 or
				&check_ip($new_ip) != 0) {

				####################################
				#	Wrong IP address specification
				####################################
				printf("Wrong IP address specification on line %s - ".
					"line skipped\n", $line);
				$were_errors = 1;
				next;
			}
			if (&check_mask($old_mask) != 0 or
				&check_mask($new_mask) != 0) {

				#################################
				#	Wrong netmask specification
				#################################
				printf("Wrong netmask specification on line %s - ".
					"line skipped\n", $line);
				$were_errors = 1;
				next;
			}

			######################################
			#	Check if addressess are the same
			######################################
			if ($old_iface eq $new_iface and
				$old_ip    eq $new_ip    and
				$old_mask  eq $new_mask) {

				#########################
				#	Avoiding duplicates
				#########################
				if (not grep(/^$new_ip$/, @skipped)) {
					push(@skipped, $new_ip);
				}

				next;
			}

			###########################################################
			#	Check that given old IP addres is registered in Plesk
			###########################################################
			my $query = 
				"SELECT COUNT(*) FROM IP_Addresses ".
				"WHERE ip_address = '$old_ip'";

			my $command = $config{MYSQL} . " -e \"$query\"";

			#########################
			#	Execute MySQL query
			#########################
			open(QUERY, "$command |");
			while (<QUERY>) {

				if (m/^\s*([\S]+)\s*$/) {

					my $used = $1;
					if ($used == 0) {

						printf("Line %s: IP address %s isn't used ".
							"in Plesk - line skipped\n", $line, $old_ip);
						$were_errors = 1;
						close(QUERY);
						next LINE;
					}

					if ($used > 1) {
						printf("Database incostince detected: \n".
							"\tone IP address (%s) is used twice\n", $old_ip);
						printf("Line %s: IP address can't be restored\n"
							, $line);
						$were_errors = 1;
						close(QUERY);
						next LINE;
					}

				} else {

					printf("Can't retrieve information about IP address ".
						"%s from Plesk database - line skipped\n", $old_ip);
					$were_errors = 1;
					close(QUERY);
					next LINE;
				}
			}
			close(QUERY);

			################################################
			#	Old and new IP targets seems to be correct
			#	check that old IP was not mapped already
			################################################
			if (exists $ips{$old_ip}) {

				printf("Line %s: IP address %s is already mapped - ".
					"skipped\n", $line, $old_ip);
				$were_errors = 1;
				next;
			}
			####################################################
			#	Check that new IP wasn't used as target before
			####################################################
			foreach my $ip_key (keys %ips) {

				my @ip = @{ $ips{$ip_key} };

				if ($ip[1] eq $new_ip) {

					printf("Line %s: Other IP address is already ".
						"mapped to %s - skipped\n", $line, $new_ip);
					$were_errors = 1;
					next LINE;
				}
			}

			#######################################################
			#	Collisions aren't found - add to list for restore
			#######################################################
			$ips{$old_ip} = [$new_iface, $new_ip, $new_mask];

		} else {

			############################################################
			#	Not a comment, not empty line and not a correct syntax
			############################################################
			printf("Can't parse line %s: not valid syntax - ".
				"line skipped\n", $line);
			$were_errors = 1;
			next;
		}
	}

	close(MAPFILE);

	#############################################
	#	If there are no IP to map - die success
	#############################################
	if (0 == scalar(keys %ips)) {
		printf("No IP addresses to map\n");
		die "Mapping is not required\n";
	}

	####################################################################
	#	Add IPs which are not presented in map file but exist in Plesk
	####################################################################
	$query =
		"SELECT ip_address FROM IP_Addresses WHERE ".
		"ip_address != '" . 
		join("' AND ip_address != '", @skipped, keys %ips) . "'";

	$command = $config{MYSQL} . " -e \"$query\"";

	open(QUERY, "$command |");
	while (<QUERY>) {

		if (m/^\s*([\S]+)\s*$/) { push(@skipped, $1); }
	}
	close(QUERY);

	######################################################################
	#	Check that there are no collisions between skipped addresses and
	#	addresses, scheduled for mapping. If such collision exists - die
	######################################################################
	foreach my $ip_key (keys %ips) {

		my @ip = @{ $ips{$ip_key} };
		my $ipaddr = $ip[1];

		if (grep(/^$ipaddr$/,@skipped)) {
			printf("Can't perform mapping.\n".
				"Error: IP address %s is specified twice as a target\n", 
				$ipaddr);
			
			#####################
			#	Mapping aborted
			#####################
			printf("Mapping procedure stopped. Correct errors and try again\n");
			return -1;
		}
	}

	###############################################################
	#	Check that there are no collisions between target IPs and 
	#	source IPs
	###############################################################
	{
		my @src_ips = keys %ips;

		foreach my $ip_key (keys %ips) {

			my @ip = @{ $ips{$ip_key} };
			my $ipaddr = $ip[1];

			if (grep(/^$ipaddr$/, @src_ips)) {
			printf("Can't perform mapping.\n".
				"Error: IP address %s is used as target and destination on ".
				"different lines of map file\n",
				$ipaddr);
			
			#####################
			#	Mapping aborted
			#####################
			printf("Mapping procedure stopped. Correct errors and try again\n");
			return -1;
			}
		}
	}
	
	#######################################################
	#	Ask user in interactive mode about errors occured
	#######################################################
	if ($were_errors) {

		printf("\n\tATTENTION!!!\n\nThere were errors in mapping procedure\n");
		printf("Here is a map, which will be restored currently:\n\n");

		my $answer;

		foreach my $old_ip (keys %ips) {

			my @new_ip = @{ $ips{$old_ip} };

			printf("%s -> %s:%s %s\n", 
				$old_ip, $new_ip[0], $new_ip[1], $new_ip[2]);
		}

		printf("\nDo you wish to restore it as is (y/n)? [n]: ");
		$answer = getc;

		if (($answer eq 'n') or ($answer eq 'N') or ($answer eq "\n")) {

			#####################
			#	Mapping aborted
			#####################
			printf("Mapping procedure stopped. Correct errors and try again\n");
			return -1;
		}
	}

	######################
	#	Ready steady go!
	######################
	printf("Checking done, now Plesk configuration will be updated.\n");

	#############################################################
	#	Flag to reconfigure apache if IP addresses were changed
	#############################################################
	my $reconfigure = 0;

	##############################
	#	Process all selected IPs
	##############################
IP:
	foreach my $old_ip (keys %ips) {

		my @new_ip_array = @{ $ips{$old_ip} };

		my $new_iface = $new_ip_array[0];
		my $new_ip    = $new_ip_array[1];
		my $new_mask  = $new_ip_array[2];

		printf("%s -> %s:%s %s\n", 
			$old_ip, $new_iface, $new_ip, $new_mask);

		##########################
		#	Get ID of IP address
		##########################
		my $id = 0;

		my $query = 
			"SELECT id FROM IP_Addresses ".
			"WHERE ip_address = '$old_ip'";

		my $command = $config{MYSQL} . " -e \"$query\"";

		#########################
		#	Execute MySQL query
		#########################
		open(QUERY, "$command |");
		while (<QUERY>) {

			if (m/^\s*([\S]+)\s*$/) {

				$id = $1;
				if ($id == 0) {

					printf("Can't map IP address %s - can't find ".
						"ID in database - mapping skipped\n", $old_ip);
					close(QUERY);
					next 2;
				}

			} else {

				printf("Can't retrieve information about IP address ".
					"%s from Plesk database - mapping skipped\n", $old_ip);
				close(QUERY);
				next 2;
			}
		}
		close(QUERY);

		################################
		#	Set new values in database
		################################
		printf("Update ip %s in database... ", $new_ip);

		#############################
		#	Change IP address value
		#############################
		$query = 
			"UPDATE IP_Addresses SET ".
			"ip_address='$new_ip', mask='$new_mask', iface='$new_iface' ".
			"WHERE id='$id'";

		$command = $config{MYSQL} . " -e \"$query\"";

		if (system($command) != 0) {
			printf("\nError during Plesk database update of IP address %s - ".
				"call support!\n", $old_ip);
			next;
		}

		#########################
		#	Change DNS settings
		#########################
		$query = 
			"UPDATE dns_recs SET ".
			"val='$new_ip', displayVal='$new_ip' WHERE val='$old_ip'";

		$command = $config{MYSQL} . " -e \"$query\"";

		if (system($command) != 0) {
			printf("\nError during Plesk database update of IP address %s - ".
				"call support!\n", $old_ip);
			next;
		}

		#############################
		#	Update reverse DNS zone
		#############################
		$query = 
			"UPDATE dns_recs SET ".
			"host='$new_ip', displayHost='$new_ip' WHERE host='$old_ip' AND type='PTR'";

		$command = $config{MYSQL} . " -e \"$query\"";

		if (system($command) != 0) {
			printf("\nError during Plesk database update of IP address %s - ".
				"call support!\n", $old_ip);
			next;
		}

		#####################################
		#	Update main host IP if required
		#####################################
		$query = 
			"UPDATE dns_recs SET host='$new_ip', displayHost='$new_ip' WHERE host='$old_ip' ".
			"AND dns_zone_id = NULL";

		$command = $config{MYSQL} . " -e \"$query\"";

		if (system($command) != 0) {
			printf("\nError during Plesk database update of IP address %s - ".
				"call support!\n", $old_ip);
			next;
		}

		$reconfigure = 1;
		printf("ok\n");

		############################################################
		#	Get domains with given IP address used in DNS settings
		############################################################
		$query = 
			"SELECT DISTINCT domains.name FROM domains, dns_zone INNER JOIN dns_recs ".
			"ON dns_recs.dns_zone_id=dns_zone.id WHERE dns_recs.val='$new_ip'";

		$command = $config{MYSQL} . " -e \"$query\"";

		open(QUERY, "$command |");
		while (<QUERY>) {

			if (m/^\s*([\S]+)\s*$/) { push(@domains, $1); }
		}
		close(QUERY);

		########################################################
		#	Get affected domains with hostings and forwardings
		########################################################
		$query = 
			"SELECT domains.name FROM domains INNER JOIN forwarding ".
			"ON domains.id=forwarding.dom_id ".
			"WHERE forwarding.ip_address_id='$id'";
		$command = $config{MYSQL} . " -e \"$query\"";

		open(QUERY, "$command |");
		while (<QUERY>) {

			if (m/^\s*([\S]+)\s*$/) { push(@domains, $1); }
		}
		close(QUERY);

		$query = 
			"SELECT domains.name FROM domains INNER JOIN hosting ".
			"ON domains.id=hosting.dom_id ".
			"WHERE hosting.ip_address_id='$id'";
		$command = $config{MYSQL} . " -e \"$query\"";

		open(QUERY, "$command |");
		while (<QUERY>) {

			if (m/^\s*([\S]+)\s*$/) { push(@domains, $1); }
		}
		close(QUERY);

		##########
		#	Uniq
		##########
		my %seen = ();
		foreach my $domain (@domains) {
			$seen{$domain}++;
		}
		@domains = keys %seen;

		########################################################
		#	Check if given IP address already exists in system
		########################################################
		open(IPLIST, $config{IFMNG} . " --interface $new_iface |") 
			or next IP;

		while (<IPLIST>) {

			if (m/^\s*([\S]+)\s+([\S]+)\s+.*$/) {
				if (($new_ip eq $1) and ($new_mask eq $2)) {

					##################################################
					#	Address already here, continue with the next
					##################################################
					close(IPLIST);
					next IP;
				}

				if ($new_ip eq $1) {

					###################################################
					#	Address already exists, but netmask is wrong
					#	delete an old one and add new correct address
					#	later
					###################################################
					if (system($config{IFMNG}." --del $new_ip") != 0) {
						printf("Can't correct netmask for IP $new_ip ".
							"(can't delete IP address in order to recreate ".
							"it later) - skipping it\n");
						next IP;
					}
				}
			}
		}

		close(IPLIST);

		##################################
		#	Update/set network interface
		##################################
		printf("Adding ip %s:%s/%s to system network interfaces... ",
			$new_iface, $new_ip, $new_mask);

		if (system($config{IFMNG} . 
			" --new $new_iface $new_ip $new_mask > /dev/null ") != 0) {
			printf("\nError during adding %s:%s/%s into system - ".
				"call support!\n", $new_iface, $new_ip, $new_mask);
			next;
		}
		printf("ok\n");
	}

	##########################################
	#	Reconfigure DNS for affected domains
	##########################################
	if (scalar(@domains) != 0) {

		printf("Reconfiguring DNS settings: \n");

		foreach my $domain (@domains) {

			printf("domain $domain... ");

			if (system($config{DNSMNG} . " update $domain > /dev/null")!=0) {
				printf("Can't reconfigure DNS settings for domain %s - ".
					"call support!\n", $domain);
				next;
			}
			printf("ok\n");
		}

		printf("Restarting DNS service... ");
		if (system($config{DNSMNG} . " restart > /dev/null") != 0) {
			printf("failed\nCan't restart DNS service");
		} else {
			printf("ok\n");
		}
	}

	if ($reconfigure) {
		printf("Now reconfigure apache... ");

		if (system($config{MYAPCIRST} . " -v -a > /dev/null") != 0) {
			printf("Can't reconfigure webserver - call support!\n");
			return -1;
		}
		printf("ok\n");
	}

	$wd = $config{PRODUCT_ROOT_D}.'/admin/bin/modules/watchdog/wd';
	if (-x $wd) {
		printf("Reconfiguring Watchdog module... ");

		if (0 != system($wd . " --adapt >/dev/null 2>&1")) {
			printf("Can't reconfigure Watchdog module - call support!\n");
			return -1;
		}
		printf("ok\n");
	}

	return 0;
}

####################################################
#	Checks given IP address for syntax correctness
####################################################
sub check_ip() {

	my ($ip) = @_;

	$ip =~ m/(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/;

	my $d1 = $1;
	my $d2 = $2;
	my $d3 = $3;
	my $d4 = $4;

	if (not defined $d1 or 
		not defined $d2 or 
		not defined $d3 or 
		not defined $d4) {

		return -1;
	}

	if (($d1 < 0 or 255 < $d1) or
		($d2 < 0 or 255 < $d2) or
		($d3 < 0 or 255 < $d3) or
		($d4 < 0 or 255 < $d4)) {

		return -1;
	}

	return 0;
}

#################################################
#	Checks given netmask for syntax correctness
#################################################
sub check_mask() {

	my ($mask) = @_;

	$mask =~ m/(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/;

	my $d1 = $1;
	my $d2 = $2;
	my $d3 = $3;
	my $d4 = $4;

	if (not defined $d1 or 
		not defined $d2 or 
		not defined $d3 or 
		not defined $d4) {

		return -1;
	}

	if (($d1 < 0 or 255 < $d1) or
		($d2 < 0 or 255 < $d2) or
		($d3 < 0 or 255 < $d3) or
		($d4 < 0 or 255 < $d4)) {

		return -1;
	}

	return 0;
}

#####################################################
#	Check that interface is presented in the system
#####################################################
sub check_iface() {

	my ($iface) = @_;

	###############################################
	#	Get list of all system network interfaces
	###############################################
	if (not defined @interfaces) {

		open(IFLIST, $config{IFMNG} . " --all |") 
			or die "Can't get system network interfaces\n";

		while (<IFLIST>) {

			if (m/^\s*([a-z][\S]+)\s*$/) { push(@interfaces, $1); }
		}

		close(IFLIST);
	}

	if (not grep(/^$iface$/, @interfaces)) {
		printf("Interface '%s' is not presented in the system\n", $iface);
		return -1;
	}

	return 0;
}

# use for quoting spec symbols in arguments during some external util executing
# stolen from bu
sub shellArgQuote($)
{
	($_) = @_;
	s/'/'\\''/g;
	return "'$_'";
}
