use psa;


# increase locale field length

ALTER TABLE clients MODIFY COLUMN locale CHAR(3) NOT NULL DEFAULT 'en';


# create table to store clients ip pool

CREATE TABLE cl_ip (
	cl_id	INT UNSIGNED NOT NULL REFERENCES clients (id)	# client id
,	ip		CHAR(15) NOT NULL							# parameter value
,		PRIMARY KEY (ip)
);


# move disk space size to domain level

ALTER TABLE domains 
	ADD COLUMN	size		BIGINT UNSIGNED DEFAULT 1048576	AFTER htype
,	ADD COLUMN	real_size	BIGINT UNSIGNED DEFAULT 0		AFTER size
	;

# create table for DNS Resource Records Templates

CREATE TABLE dns_recs_t (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY			# DNS RR ID
,	type	ENUM('NS','A','CNAME','MX','PTR','none') NOT NULL DEFAULT 'A'	# RR type
,	host	VARCHAR(255) NOT NULL							# left column in zone file
,	val		VARCHAR(255) NOT NULL							# right column in zone file
,	opt		VARCHAR(255) NOT NULL DEFAULT ''				# options (preferences for MX, mask for PTR)
,	time_stamp	TIMESTAMP(14) NOT NULL
);

# insert default DNS template

replace into dns_recs_t(host, type, val)		values("<domain>.", "NS", "ns.<domain>");
replace into dns_recs_t(host, type, val)		values("ns.<domain>.", "A", "<ip>");
replace into dns_recs_t(host, type, val)		values("<domain>.", "A", "<ip>");
replace into dns_recs_t(host, type, val)		values("webmail.<domain>.", "A", "<ip>");
replace into dns_recs_t(host, type, val, opt)	values("<domain>.", "MX", "mail.<domain>.", 10);
replace into dns_recs_t(host, type, val)		values("mail.<domain>.", "CNAME", "<domain>.");
replace into dns_recs_t(host, type, val)		values("ftp.<domain>.", "CNAME", "<domain>.");
replace into dns_recs_t(host, type, val, opt)	values("<ip>", "PTR", "<domain>.", 24);


# increase login/password for system users

ALTER TABLE hosting
	MODIFY COLUMN	login	CHAR(16) NOT NULL
,	MODIFY COLUMN	passwd	CHAR(16) NOT NULL
,	ADD COLUMN		asp		ENUM('false','true') NOT NULL DEFAULT 'false'		AFTER perl
,	ADD COLUMN		webstat		ENUM('false','true') NOT NULL DEFAULT 'false'	AFTER mysql
	;


# change column path to binary, to disting case

ALTER TABLE protected_dirs
	MODIFY COLUMN	path	VARCHAR(245) BINARY NOT NULL
	;


# change column login to binary, to disting case

ALTER TABLE pd_users
	MODIFY COLUMN	login	CHAR(20) BINARY NOT NULL
	;


# add column mbox_quota

ALTER TABLE mail
	ADD COLUMN	mbox_quota	BIGINT DEFAULT '-1' NOT NULL AFTER autoresponder
	;
	

# change default value for column ans_freq

ALTER TABLE mail_resp
	ALTER COLUMN	ans_freq	SET DEFAULT '86400'
	;
	

# add column num to sessions table

ALTER TABLE sessions
	ADD COLUMN	num		INT UNSIGNED NOT NULL UNIQUE AUTO_INCREMENT	FIRST
	;
	
CREATE TABLE session_params (
	session_num		INT UNSIGNED NOT NULL 	# session number for linking with session_params
,	param			VARCHAR(100) NOT NULL 	# session parameter name
,	val				VARCHAR(255) NOT NULL	# session parameter value
,	PRIMARY KEY		(session_num, param)
);


# change column username in web_users table

ALTER TABLE web_users
	MODIFY COLUMN	username	CHAR(16) NOT NULL
,	ADD COLUMN		ssi		ENUM('false','true') NOT NULL DEFAULT 'false'
,	ADD COLUMN		php		ENUM('false','true') NOT NULL DEFAULT 'false'
,	ADD COLUMN		cgi		ENUM('false','true') NOT NULL DEFAULT 'false'
,	ADD COLUMN		perl	ENUM('false','true') NOT NULL DEFAULT 'false'
,	ADD COLUMN		asp		ENUM('false','true') NOT NULL DEFAULT 'false'
	;


#	domain level users (dom_level_usrs) table
CREATE TABLE dom_level_usrs (
	dom_id		INT UNSIGNED PRIMARY KEY REFERENCES domains (id)# domain ID
,	state		ENUM('false','true') NOT NULL DEFAULT 'false'	# domain user status from client
,	passwd		VARCHAR(20)  NOT NULL							# DL user password
);


# anonymous virtual FTP domain information
CREATE TABLE anon_ftp (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
,	dom_id		INT UNSIGNED NOT NULL REFERENCES domains (id)
,	server_name	VARCHAR(255) NOT NULL							
,	max_conn	INT UNSIGNED									# maximum connections limit
,	bandwidth	INT UNSIGNED									# bandwidth limit
,	incoming	ENUM('false','true') NOT NULL DEFAULT 'false'	# Turn incoming on or off
,	status		ENUM('false','true') NOT NULL DEFAULT 'false'	# Turn anonymous virtual FTP domain on or off
,	quota       BIGINT UNSIGNED DEFAULT 1048576					# Incoming directory quota
,               UNIQUE(dom_id)
);


CREATE TABLE data_bases (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
,	name	CHAR(32) BINARY NOT NULL
,		UNIQUE (name)
,	type		ENUM('mysql','postgres') NOT NULL DEFAULT 'mysql'
,	dom_id	INT UNSIGNED NOT NULL REFERENCES domains (id)
);


CREATE TABLE db_users (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
,	login	CHAR(16) BINARY NOT NULL
,	passwd	CHAR(16) NOT NULL
,	db_id	INT UNSIGNED NOT NULL REFERENCES protected_dirs (id)
,		UNIQUE (db_id,login)
);

CREATE TABLE db_params (
	cl_login	CHAR(255) NOT NULL PRIMARY KEY
,	db_name		CHAR(32) NOT NULL REFERENCES data_bases (name)
,	login		CHAR(16) NOT NULL REFERENCES db_users (login)
,	passwd		CHAR(16) NOT NULL REFERENCES db_users (passwd)
);


#	list supported interface languages
CREATE TABLE locales (
	id			VARCHAR(3)	PRIMARY KEY         # lang ID: "en", "it", etc
,	lang		VARCHAR(30)						# lang name "English", "Japanese", etc
,		UNIQUE(id)
);
	


replace into locales (id, lang) values('en', 'English');

replace into misc (param,val) values('version', '0200');

replace into misc (param,val) values('locale', 'en');
replace into misc (param,val) values('def_locale', 'en');

replace into misc (param,val) values('cl_sort', '0');
replace into misc (param,val) values('dom_sort', '0');

replace into misc (param,val) values('psa_configured', 'true');

delete from misc where param='admin_isPrefEdit';
