use psa;


###DNS

# Master DNS servers
CREATE TABLE dns_masters (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY			# DNS RR ID
,	dom_id		INT UNSIGNED NOT NULL REFERENCES domains (id)	# domain ID
,		INDEX (dom_id)
,	ip_address	VARCHAR(255) NOT NULL							# IPs of master DNS servers
,		UNIQUE(dom_id, ip_address)
);

# Add reference to dns table
ALTER TABLE domains
	ADD COLUMN dns_type		ENUM('slave','master') NOT NULL DEFAULT 'master'	# dns type for domain
,	ADD INDEX (cl_id)
	;

# Add reference to dns table
ALTER TABLE dns_recs
	ADD INDEX (type)
	;

#-- Begin DNS upgrade
update domains set dns_type="master";
#-- End DNS upgrade

###!DNS

###SysUsers
CREATE TABLE sys_users (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY	# sys_user ID, autogenerated
,	login		VARCHAR(20)  NOT NULL					# system user login
,		UNIQUE (login)									# unique index on login
,	passwd		VARCHAR(20)  NOT NULL					# system user password
,	home		VARCHAR(255) NOT NULL					# system user home directory
,	shell		VARCHAR(255) NOT NULL					# system user shell
,	quota		BIGINT UNSIGNED DEFAULT 0				# disk space hard limitation(MBytes)
);

# Add reference to sys_user table
ALTER TABLE hosting
	ADD COLUMN sys_user_id		INT UNSIGNED NOT NULL REFERENCES sys_users (id)	AFTER dom_id
,	ADD COLUMN ip_address	CHAR(15) NOT NULL AFTER sys_user_id
,	ADD COLUMN fp_ssl		ENUM('false','true') NOT NULL DEFAULT 'false' AFTER fp
	;

# Add reference to sys_user table
ALTER TABLE web_users
	ADD COLUMN sys_user_id		INT UNSIGNED NOT NULL REFERENCES sys_users (id)	AFTER dom_id
,	ADD INDEX (dom_id)
	;

#-- Begin sys_user upgrade
insert into sys_users (login, passwd)
           select login, passwd from hosting;
insert into sys_users (login)
           select username from web_users;
#-- End sys_user upgrade (completely in shell)

# Remove needed columns exclude username
ALTER TABLE hosting
	DROP COLUMN passwd
	;

# Remove needed columns exclude username
ALTER TABLE web_users
	DROP INDEX username
	;

###!SysUsers

###Cards

# add reference to Cards
ALTER TABLE  dom_level_usrs
	ADD COLUMN card_id		INT UNSIGNED REFERENCES Cards (id)	AFTER passwd
	;

#Remove MPC fields
ALTER TABLE clients
	DROP COLUMN mpc_user
,	DROP COLUMN mpc_pass
	;

# Cards table
CREATE TABLE Cards (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY	# ID, autogenerated
,	personalName	VARCHAR(255) NOT NULL		# personal name
,	companyName		VARCHAR(255)				# company name
,	phone			VARCHAR(255)				# contact phone
,	fax				VARCHAR(255)				# contact fax
,	email			VARCHAR(255)				# contact e-mail
,	address			VARCHAR(255)				# address
,	city			VARCHAR(255)				# city
,	state			VARCHAR(255)				# state
,	zip				CHAR(10)					# postoffice code
,	country			CHAR(2)						# country
);

#-- Begin Cards upgrade

#-- End Cards upgrade

###!Cards


#-- domains disk usage information
CREATE TABLE disk_usage (
	dom_id	INT UNSIGNED NOT NULL REFERENCES domains (id)	-- domain ID
,	httpdocs	BIGINT UNSIGNED DEFAULT 0		-- space used by http documents
,	httpsdocs	BIGINT UNSIGNED DEFAULT 0		-- space used by https documents
,	anonftp		BIGINT UNSIGNED DEFAULT 0		-- space used by anonymous ftp directory
,	logs		BIGINT UNSIGNED DEFAULT 0		-- space used by domain logs directory
,	dbases		BIGINT UNSIGNED DEFAULT 0		-- space used by domain databases
,	mailboxes	BIGINT UNSIGNED DEFAULT 0		-- space used by domain mail boxes
,	PRIMARY KEY (dom_id)
);


# Change UNIQUE
ALTER TABLE protected_dirs
	DROP INDEX path
,	ADD UNIQUE(dom_id,path)
	;

# Add pub_key
ALTER TABLE certificates
	ADD COLUMN pub_key		BLOB AFTER pvt_key
,	ADD INDEX (dom_id)
	;

# Change responce frequency + add reply_to
ALTER TABLE mail_resp
	ADD COLUMN reply_to	VARCHAR(255) AFTER subject
,	ALTER COLUMN ans_freq SET DEFAULT 10
	;

#-- Begin mail_resp upgrade
update mail_resp	set ans_freq=10;
#-- End mail_resp upgrade

# Add number of responses from email in this day
ALTER TABLE resp_freq
	ADD COLUMN num			INT DEFAULT 0 AFTER rn_id
	;

-- Notifications table
CREATE TABLE Notifications (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY	-- Notification ID
,	status		CHAR(1)	NOT NULL								-- Notification status (on/off)
,	send2admin	CHAR(1)	NOT NULL								-- Send notification to admin ?
,	send2client	CHAR(1)	NOT NULL								-- Send notification to client ?
,	send2dlu	CHAR(1)	NOT NULL								-- Send notification to domain level user ?
,	subj		VARCHAR(255) NOT NULL							-- Notification subject field
,	note_id		INT	UNSIGNED REFERENCES Notes (id)		-- Respected note ID
);

-- Note table
CREATE TABLE Notes (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY	-- Note ID
,	text	TEXT	NOT NULL							-- Note text
);

# mail alias table
CREATE TABLE mail_aliases (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY			# ID
,	mn_id		INT UNSIGNED NOT NULL REFERENCES mail (id)		# mailname ID
,	alias		VARCHAR(245) NOT NULL							# mail alias name
,		UNIQUE(mn_id,alias)
);

# Add INDEX
ALTER TABLE smtp_poplocks
	ADD INDEX (ip_address)
	;

CREATE TABLE log_rotation (
	id						INT UNSIGNED AUTO_INCREMENT PRIMARY KEY				# log rotation record ID, autogenerated
,	period_type				ENUM('by_time','by_size') NOT NULL DEFAULT 'by_time'# type of log rotation period
,	period					VARCHAR(50)											# log rotation period
,	max_number_of_logfiles	BIGINT UNSIGNED DEFAULT 0							# max number of logfiles
,	compress_enable			ENUM('false','true') NOT NULL DEFAULT 'false'		# enable or disable logfiles compression
,	email					VARCHAR(255)										# email address for rotated logfile sending
,	turned_on				ENUM('false','true') NOT NULL DEFAULT 'false'		# enable or disable log rotation service
);

-- permissions table
CREATE TABLE Permissions (
	id			INT UNSIGNED AUTO_INCREMENT NOT NULL
,	permission	CHAR(255) NOT NULL
,	value		CHAR(255)
,		PRIMARY KEY (id, permission)
);

-- parameters table
CREATE TABLE Parameters (
	id			INT UNSIGNED AUTO_INCREMENT NOT NULL
,	parameter	CHAR(255) NOT NULL
,	value		CHAR(255)
,		PRIMARY KEY (id, parameter)
);

-- limits table
CREATE TABLE Limits (
	id			INT UNSIGNED AUTO_INCREMENT NOT NULL
,	limit_name	CHAR(255) NOT NULL
,	value		CHAR(255)
,		PRIMARY KEY (id, limit_name)
);

-- IP pool
RENAME TABLE cl_ip to IPPool;
ALTER TABLE IPPool DROP PRIMARY KEY;
ALTER TABLE IPPool CHANGE cl_id id INT UNSIGNED NOT NULL;
ALTER TABLE IPPool ADD PRIMARY KEY (id, ip);
ALTER TABLE IPPool MODIFY id INT UNSIGNED AUTO_INCREMENT NOT NULL;
ALTER TABLE IPPool ADD UNIQUE (ip);
INSERT INTO cl_param SELECT id, 'pool_id', id from IPPool group by id;

# Logos tables
CREATE TABLE Logos (
        id              INT UNSIGNED AUTO_INCREMENT PRIMARY KEY # ID
,       name            VARCHAR(255)
,       fake            VARCHAR(255)
,       url             VARCHAR(255)
,       showMode        VARCHAR(255)
);

# template table
CREATE TABLE Templates (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY	# -- tmpl ID, autogenerated
,	name		CHAR(255) NOT NULL
,	note_id		INT	UNSIGNED REFERENCES Notes (id)	# Respected note ID
);

# template data table
CREATE TABLE TmplData (
	tmpl_id		INT UNSIGNED NOT NULL
,	element		CHAR(255) NOT NULL
,	value		CHAR(255) NOT NULL
,	PRIMARY KEY (tmpl_id, element)
);

# --- Identifiers repository
CREATE TABLE Repository (
	rep_id			INT UNSIGNED AUTO_INCREMENT NOT NULL
,	component_id	INT UNSIGNED NOT NULL
,	PRIMARY KEY (rep_id, component_id)
);

# skins table
CREATE TABLE Skins (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY	-- skin ID, autogenerated
,	name	VARCHAR(255) NOT NULL		-- name of skin
,	place	CHAR(255) NOT NULL			-- skin directory
);

-- slave hosts
CREATE TABLE slaves (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY -- ID
,	hostname	VARCHAR(255) NOT NULL
,	UNIQUE(hostname)
);

CREATE TABLE slave_params (
	slave_id	INT UNSIGNED NOT NULL REFERENCES slaves (id)
,	parameter	VARCHAR(255) NOT NULL
,	value		VARCHAR(255) NOT NULL
,		PRIMARY KEY(slave_id, parameter)
);

# Client-mailer queries list
CREATE TABLE mailer_queries (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY # ID
,	query		TEXT NOT NULL
,	description	VARCHAR(100)  NOT NULL
,	locale_key	VARCHAR(20)   NOT NULL DEFAULT ""
);

# -- Custom buttons (fill by admin)
CREATE TABLE custom_buttons (
	id			INT NOT NULL AUTO_INCREMENT PRIMARY KEY
,	image_path	VARCHAR(255)
,	url			VARCHAR(255)
,	conhelp		TEXT
);

# -- Interfaces lists parameters (page, sort, filter)
CREATE TABLE ListsParams (
	user_id			INT UNSIGNED NOT NULL DEFAULT 0		-- current session user id (client id now)
,	parentList_id	INT	UNSIGNED NOT NULL DEFAULT 0		-- list owner id (0 for domain independent lists)
,	list_name	VARCHAR(255) NOT NULL DEFAULT ''	-- list class name (ex: DomainsList)
,	sort		VARCHAR(255) NOT NULL DEFAULT 'name'-- column name for sorting (ex: name[_reverse])
,	filter		VARCHAR(255) NOT NULL DEFAULT ''	-- search filter
,	page		INT			 NOT NULL DEFAULT 0	-- current page number
,		PRIMARY KEY (user_id, parentList_id, list_name)
);

# -- PSA Upgrade history
CREATE TABLE upgrade_history (
	upgrade_date DATETIME
,	version_info VARCHAR(50)
,	db_version   VARCHAR(8)
);

replace into mailer_queries (query,description) values ("select email from clients","all clients");
replace into misc (param,val) values('version', '0500');

delete from Notifications;
replace into Notifications (id, note_id, subj, send2admin, send2client) values (1, 1, 'Notification of the client creation.', 1, 1);
replace into Notifications (id, note_id, subj, send2admin, send2client) values (2, 2, 'Notification of the domain creation.', 1, 1);
replace into Notifications (id, note_id, subj, send2admin, send2client) values (3, 3, 'Notification of problem domains', 1, 1);

delete from Notes;
replace into Notes (id, text) values (1, "This is a notification of new client creation from PSA.\nClient login: <client>\nClient password: <password>\nPSA entry point: https://<ip>:8443");
replace into Notes (id, text) values (2, "This is a notification of new domain creation from PSA.\nDomain name: <domain>\nClient owner: <client>\nDomain IP: <ip>");
replace into Notes (id, text) values (3, "\nLimit has been exceeded on domain '<domain>' owned by client '<client_login>' (Contact name '<client_contact_name>'):\nDisk space usage: <disk_usage>\nDisk space limit: <disk_space_limit>\nTraffic: <traffic>\nTraffic limit: <traffic_limit>\n");

replace into misc (param,val) values('include_logs', 'false');
replace into misc (param,val) values('include_databases', 'true');
replace into misc (param,val) values('include_mailboxes', 'false');

replace into misc (param,val) values('slave_request_timeout', '20');

delete from misc where param='mbox_quota';

delete from misc where param='not_showLicense';

delete from misc where
param='cl_filter' or param='cl_sort' or param='cl_page' or
param='dom_filter' or param='dom_sort' or param='dom_page' or
param='dbusr_filter' or param='dbusr_sort' or param='dbusr_page' or
param='db_filter' or param='db_sort' or param='db_page' or
param='mn_filter' or param='mn_sort' or param='mn_page' or
param='pdusr_filter' or param='pdusr_sort' or param='pdusr_page' or
param='pd_filter' or param='pd_sort' or param='pd_page' or
param='wu_filter' or param='wu_sort' or param='wu_page';

delete from cl_param where
param='cl_filter' or param='cl_sort' or param='cl_page' or
param='dom_filter' or param='dom_sort' or param='dom_page' or
param='dbusr_filter' or param='dbusr_sort' or param='dbusr_page' or
param='db_filter' or param='db_sort' or param='db_page' or
param='mn_filter' or param='mn_sort' or param='mn_page' or
param='pdusr_filter' or param='pdusr_sort' or param='pdusr_page' or
param='pd_filter' or param='pd_sort' or param='pd_page' or
param='wu_filter' or param='wu_sort' or param='wu_page';

delete from dom_param where
param='cl_filter' or param='cl_sort' or param='cl_page' or
param='dom_filter' or param='dom_sort' or param='dom_page' or
param='dbusr_filter' or param='dbusr_sort' or param='dbusr_page' or
param='db_filter' or param='db_sort' or param='db_page' or
param='mn_filter' or param='mn_sort' or param='mn_page' or
param='pdusr_filter' or param='pdusr_sort' or param='pdusr_page' or
param='pd_filter' or param='pd_sort' or param='pd_page' or
param='wu_filter' or param='wu_sort' or param='wu_page';

