use psa;

-- Task 2.1. Support more than one IP for Name-based Domains task (and + Support mod_python) --

CREATE table IP_Addresses (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY		-- IP Address ID
,	ip_address	char(15) NOT NULL UNIQUE	-- IP Address number in a dot notation
,	mask		char(15) NOT NULL DEFAULT '255.255.255.0'
,	iface		VARCHAR(20) NOT NULL
,	type		ENUM('shared','exclusive') NOT NULL DEFAULT 'shared'	-- IP Address type: shared or exclusive
,	ssl_certificate_id	INT UNSIGNED NOT NULL REFERENCES certificates (id)	-- SSL Certificate ID
,	default_domain_id	INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES hosting (dom_id)	-- default hosting ID (really dom_id)
);

ALTER TABLE hosting
	ADD COLUMN ip_address_id INT UNSIGNED NOT NULL REFERENCES IP_Addresses(id) after ip_address
,	ADD COLUMN python			ENUM('false','true') NOT NULL DEFAULT 'false' after perl	-- + Support mod_python
	;

ALTER TABLE forwarding
	ADD COLUMN ip_address_id INT UNSIGNED NOT NULL REFERENCES IP_Addresses (id)
	;

-- Task 2.8. SSL certificate management

ALTER TABLE certificates
	ADD COLUMN name VARCHAR(255)
,	ADD COLUMN cert BLOB
,	ADD COLUMN cert_file VARCHAR(255)
,	ADD COLUMN csr  BLOB
,	ADD COLUMN ca_cert BLOB
,	ADD COLUMN ca_file VARCHAR(255)
	;

ALTER TABLE domains
	ADD COLUMN cert_rep_id INT UNSIGNED DEFAULT 0 REFERENCES Repository (rep_id)	-- certificate repository ID
	;

-- Task 2.9. PSA Action Logging

CREATE TABLE actions (
     id      INT UNSIGNED AUTO_INCREMENT PRIMARY KEY		-- Action ID
,    name    VARCHAR(255) NOT NULL UNIQUE					-- Action Name
,    descr   VARCHAR(255) NOT NULL							-- Action Description
,    enabled ENUM ('false', 'true') NOT NULL default 'true'	-- Status: enabled/disabled
);

CREATE table log_actions (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY				-- Action ID
,	date		DATETIME					 					-- Action date
,	ip_address	CHAR(15) NOT NULL				 				-- IP from which was established connection
,	user		VARCHAR(255) NOT NULL				 			-- Control Panel user login name
,	action_id	INT UNSIGNED NOT NULL REFERENCES actions(id) 	-- Action ID
,	object_id	INT UNSIGNED NOT NULL 		-- Identifies operated object (interaction with BS)
);

CREATE table log_components (
	action_id	INT UNSIGNED NOT NULL REFERENCES log_actions (id)	-- Action ID
,	component	VARCHAR(255) NOT NULL								-- Component
,	old_value	VARCHAR(255) NOT NULL								-- Old value
,	new_value	VARCHAR(255) NOT NULL								-- New value
);

delete from actions;
replace into actions(name, descr, enabled) values ('client_create', 'Create Client', 'true');
replace into actions(name, descr, enabled) values ('client_update', 'Update Client', 'true');
replace into actions(name, descr, enabled) values ('client_delete', 'Delete Client', 'true');
replace into actions(name, descr, enabled) values ('domain_create', 'Create Domain', 'true');
replace into actions(name, descr, enabled) values ('domain_update', 'Update Domain', 'true');
replace into actions(name, descr, enabled) values ('domain_delete', 'Delete Domain', 'true');
replace into actions(name, descr, enabled) values ('phys_hosting_create', 'Create Physical Hosting', 'true');
replace into actions(name, descr, enabled) values ('phys_hosting_update', 'Update Physical Hosting', 'true');
replace into actions(name, descr, enabled) values ('phys_hosting_delete', 'Delete Physical Hosting', 'true');
replace into actions(name, descr, enabled) values ('mailname_create', 'Create Mailname', 'true');
replace into actions(name, descr, enabled) values ('mailname_update', 'Update Mailname', 'true');
replace into actions(name, descr, enabled) values ('mailname_delete', 'Delete Mailname', 'true');
replace into actions(name, descr, enabled) values ('webuser_create', 'Create WebUser', 'true');
replace into actions(name, descr, enabled) values ('webuser_update', 'Update WebUser', 'true');
replace into actions(name, descr, enabled) values ('webuser_delete', 'Delete WebUser', 'true');
replace into actions(name, descr, enabled) values ('client_limits_update', 'Update Client Limits', 'true');
replace into actions(name, descr, enabled) values ('domain_limits_update', 'Update Domain Limits', 'true');
replace into actions(name, descr, enabled) values ('maillist_create', 'Create Mail list', 'true');
replace into actions(name, descr, enabled) values ('maillist_update', 'Update Mail list', 'true');
replace into actions(name, descr, enabled) values ('maillist_delete', 'Delete Mail list', 'true');
replace into actions(name, descr, enabled) values ('cp_user_login', 'CP User Login', 'true');
replace into actions(name, descr, enabled) values ('cp_user_logout', 'CP User Logout', 'true');
replace into actions(name, descr, enabled) values ('dl_user_update', 'Domain User Update', 'true');

replace into misc (param,val) values('actionlog_rot_type', 			'by_period');
replace into misc (param,val) values('actionlog_rot_period', 		'day');
replace into misc (param,val) values('actionlog_rot_num_periods', 	'30');
replace into misc (param,val) values('actionlog_rot_num_records', 	'3000');
replace into misc (param,val) values('slave_services_list_with_non_obligatory_status_checking', 'tomcat');
replace into misc (param,val) values('master_services_list_for_obligatory_status_checking', 'imap,pop3,qmail,apache,ftp,dns,postgresql');


delete from misc where param='admin_isPrefEdit';
DELETE FROM misc WHERE param='admin_isMPCsyn';
DELETE FROM misc WHERE param='error_docs_format';

-- Task 2.12. Mailman support

CREATE table MailLists (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY		-- MailList ID, autogenerated
,	dom_id	INT UNSIGNED NOT NULL REFERENCES domains (id)		-- Domain ID, referense to domain id
,	name	VARCHAR(255) NOT NULL					-- MailList name
,	status	BIGINT UNSIGNED NOT NULL DEFAULT 0			-- MailList status bitmask
);

-- Task: 2.16. PSA Event Manager

CREATE TABLE event_handlers (
        id              INT UNSIGNED AUTO_INCREMENT PRIMARY KEY         -- event handler ID
,       action_id       INT UNSIGNED NOT NULL REFERENCES actions (id)   -- action ID
,       priority        INT DEFAULT 0                                   -- priority
,       user            VARCHAR(255) DEFAULT 'nobody'                   -- system user name
,       command         TEXT                                    		-- handler
);

-- Task: 2.10. PSA Text Mode

ALTER TABLE custom_buttons ADD COLUMN text VARCHAR(255);
UPDATE custom_buttons SET text=SUBSTRING_INDEX(SUBSTRING_INDEX(image_path, '/', -1), '.', 1);
ALTER TABLE custom_buttons DROP COLUMN image_path;

-- Task: 2.5. Sending Notifications on arbirary email
ALTER TABLE Notifications ADD COLUMN send2email	CHAR(1)	NOT NULL;		-- Send notification to the email ?
ALTER TABLE Notifications ADD COLUMN email		VARCHAR(255) NOT NULL;	-- Email address

-- Task: 2.11. Client-Domain Expiration (status)

-- 1) 'domains' table
ALTER  TABLE domains ADD COLUMN new_status BIGINT UNSIGNED NOT NULL DEFAULT 0;
UPDATE domains SET new_status = 64*(status != 'true'); -- STATUS_ADMIN=16, STATUS_CLIENT=64
ALTER  TABLE domains DROP COLUMN status;
ALTER  TABLE domains CHANGE COLUMN new_status status BIGINT UNSIGNED NOT NULL DEFAULT 0;

-- 2) 'clients' table
ALTER  TABLE clients ADD COLUMN new_status BIGINT UNSIGNED NOT NULL DEFAULT 0;
UPDATE clients SET new_status = 16*(status != 'true'); -- STATUS_ADMIN=16, STATUS_CLIENT=64
ALTER  TABLE clients DROP COLUMN status;
ALTER  TABLE clients CHANGE COLUMN new_status status BIGINT UNSIGNED NOT NULL DEFAULT 0;

-- Task: 2.11. Client-Domain Expiration (corresponded IDs)

ALTER  TABLE domains ADD COLUMN	limits_id	INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES Limits (id);			-- Limits object ID
ALTER  TABLE domains ADD COLUMN	params_id	INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES Parameters (id);		-- Preferences object ID

ALTER  TABLE clients ADD COLUMN limits_id	INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES Limits (id);			-- Limits object ID
ALTER  TABLE clients ADD COLUMN params_id	INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES Preferences (id);	-- Preferences object ID
ALTER  TABLE clients ADD COLUMN perm_id		INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES Permissions (id);	-- Permissions object ID
ALTER  TABLE clients ADD COLUMN pool_id		INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES Repository (rep_id);	-- Repository object ID (IP Pool)
ALTER  TABLE clients ADD COLUMN logo_id		INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES Logos (id);			-- Client's logo
ALTER  TABLE clients ADD COLUMN tmpl_id		INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES Repository (rep_id);	-- Repository object ID (Domain Templates List)

-- Task: 2.11. New notifications
replace into Notifications (id, note_id, subj, send2admin, send2client) values (4, 4, 'Client account event notification: The period of validity has expired.', 1, 1);
replace into Notifications (id, note_id, subj, send2admin, send2client) values (5, 5, 'Client account event notification: The period of validity is about to expire.', 1, 1);
replace into Notifications (id, note_id, subj, send2admin, send2client) values (6, 6, 'Domain event notification: The period of validity has expired.', 1, 1);
replace into Notifications (id, note_id, subj, send2admin, send2client) values (7, 7, 'Domain event notification: The period of validity is about to expire.', 1, 1);

UPDATE Notes SET text=REPLACE(text, '<ip>', '<hostname>') WHERE id='1';
UPDATE Notes SET text=REPLACE(text, '<client>', '<client_login>') WHERE id='1';
replace into Notes (id, text) values (4, "The period of validity of the client '<client_login>' (Contact Name '<client_contact_name>') has expired.\nThe client and all of his/her domains were deactivated on <expiration_date>.");
replace into Notes (id, text) values (5, "The period of validity of the client '<client_login>' (Contact Name '<client_contact_name>') is about to expire.\nThe client and all of his/her domains will be deactivated by <expiration_date> unless the period of validity is prolonged.");
replace into Notes (id, text) values (6, "The period of validity of the domain '<domain_name>' owned by client '<client_login>' (Contact name '<client_contact_name>') has expired.\nThe domain and all of its services were deactivated on <expiration_date>.");
replace into Notes (id, text) values (7, "The period of validity of the domain '<domain_name>' owned by client '<client_login>' (Contact name '<client_contact_name>') is about to expire.\nThe domain and all of its services will be deactivated by <expiration_date> unless the period of validity is prolonged.");

--
ALTER  TABLE domains DROP COLUMN last_mod;

-- Task: 2.18. Ability to change mail options for mail users

ALTER  TABLE mail ADD COLUMN cp_access ENUM('true','false') NOT NULL DEFAULT 'true';    -- control panel access

-- Task: 2.15. Ability to have more than one client logged into the interface at a time

DROP TABLE session_params;
DROP TABLE sessions;		-- table columns were completely changed
CREATE TABLE sessions (
	sess_id		CHAR(33)	PRIMARY KEY			-- cookie value
,	login		VARCHAR(50)	NOT NULL
,	exp_date	BIGINT UNSIGNED NOT NULL
);


-- Task: 2.2. Support both SSL and non SSL protected directory at the same time

ALTER TABLE protected_dirs ADD COLUMN non_ssl ENUM('false','true') NOT NULL DEFAULT 'false';
UPDATE protected_dirs SET non_ssl='true' WHERE `ssl`='false';

-- Task: 2.14. Tomcat

CREATE TABLE DomainServices (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY				-- domain service ID
,	dom_id		INT UNSIGNED NOT NULL REFERENCES domains (id)	-- domain ID
,	type		ENUM('mail', 'tomcat', 'maillists')							-- domain service type
,	status		BIGINT(20)											-- domain service status (on/off)
,	parameters_id	INT UNSIGNED NOT NULL REFERENCES Parameters (id)	-- parameters object ID
);

-- Task 2.19. Multiserver support orientation
INSERT INTO DomainServices (dom_id, type, status, parameters_id) SELECT id, 'mail', 0, 0 FROM domains;

CREATE TABLE WebApps (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY					-- webapp ID
,	name			VARCHAR(255)										-- webapp name
,	status			INT(10)												-- webapp status (on/off)
,	domain_service_id	INT UNSIGNED NOT NULL REFERENCES DomainServices (id)	-- domain service object ID
);

ALTER TABLE disk_usage ADD COLUMN webapps	BIGINT UNSIGNED DEFAULT 0;
ALTER TABLE disk_usage ADD COLUMN maillists	BIGINT UNSIGNED DEFAULT 0;

-- Task: Detailed traffic counting

-- daily traffic statistics for Domains
CREATE TABLE DomainsTraffic (
	dom_id		INT UNSIGNED NOT NULL REFERENCES domains (id)	-- domain ID
,	date		DATE NOT NULL			-- date
,	http_in		BIGINT UNSIGNED NOT NULL DEFAULT 0	-- http(s) input traffic in bytes for this data
,	http_out	BIGINT UNSIGNED NOT NULL DEFAULT 0	-- http(s) output traffic in bytes for this data
,	ftp_in		BIGINT UNSIGNED NOT NULL DEFAULT 0	-- ftp(s) input traffic in bytes for this data
,	ftp_out		BIGINT UNSIGNED NOT NULL DEFAULT 0	-- ftp(s) output traffic in bytes for this data
,	smtp_in		BIGINT UNSIGNED NOT NULL DEFAULT 0	-- smtp(s) input traffic in bytes for this data
,	smtp_out	BIGINT UNSIGNED NOT NULL DEFAULT 0	-- smtp(s) output traffic in bytes for this data
,	pop3_imap_in	BIGINT UNSIGNED NOT NULL DEFAULT 0	-- pop3(s)/imap(s) input traffic in bytes for this data
,	pop3_imap_out	BIGINT UNSIGNED NOT NULL DEFAULT 0	-- pop3(s)/imap(s) output traffic in bytes for this data
,		PRIMARY KEY (dom_id, date)
);

-- daily traffic statistics for Clients
CREATE TABLE ClientsTraffic (
	cl_id		INT UNSIGNED NOT NULL REFERENCES clients (id)	-- client id
,	date		DATE NOT NULL			-- date
,	http_in		BIGINT UNSIGNED NOT NULL DEFAULT 0	-- http(s) input traffic in bytes for this data
,	http_out	BIGINT UNSIGNED NOT NULL DEFAULT 0	-- http(s) output traffic in bytes for this data
,	ftp_in		BIGINT UNSIGNED NOT NULL DEFAULT 0	-- ftp(s) input traffic in bytes for this data
,	ftp_out		BIGINT UNSIGNED NOT NULL DEFAULT 0	-- ftp(s) output traffic in bytes for this data
,	smtp_in		BIGINT UNSIGNED NOT NULL DEFAULT 0	-- smtp(s) input traffic in bytes for this data
,	smtp_out	BIGINT UNSIGNED NOT NULL DEFAULT 0	-- smtp(s) output traffic in bytes for this data
,	pop3_imap_in	BIGINT UNSIGNED NOT NULL DEFAULT 0	-- pop3(s)/imap(s) input traffic in bytes for this data
,	pop3_imap_out	BIGINT UNSIGNED NOT NULL DEFAULT 0	-- pop3(s)/imap(s) output traffic in bytes for this data
,		PRIMARY KEY (cl_id, date)
);

-- Unknown task: Interfaces lists parameters expansion

ALTER TABLE ListsParams ADD COLUMN user_type INT UNSIGNED NOT NULL DEFAULT 0 AFTER user_id;		-- current session user type
ALTER TABLE ListsParams ADD COLUMN flags INT UNSIGNED NOT NULL DEFAULT 0;						-- list optionsl bitmask bits from 0 to 4 - optional flags, bits from 5 - expand status of columns
ALTER TABLE ListsParams DROP PRIMARY KEY;														-- drop old index
ALTER TABLE ListsParams ADD PRIMARY KEY (user_id, user_type, parentList_id, list_name);			-- create new index


CREATE TABLE mn_param (
	mn_id	INT UNSIGNED NOT NULL REFERENCES mail (id)	-- mail name id
,	param	VARCHAR(245) NOT NULL						-- parameter name
,	val		VARCHAR(255)								-- parameter value
,	PRIMARY KEY (mn_id,param)
);

ALTER TABLE web_users
	ADD COLUMN python			ENUM('false','true') NOT NULL DEFAULT 'false' after perl	-- + Support mod_python
;

-- db_params table is deprecated now

DROP TABLE db_params;

replace into misc (param,val) values ('latest_apache_restart', NOW());
replace into misc (param,val) values ('restart_apache_interval', 900);

-- Task N.nn. Passwords
CREATE TABLE accounts (
  id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY			-- ID, autogenerated
,	type		ENUM('plain','crypt') NOT NULL DEFAULT 'plain'		-- password type
,	password	VARCHAR(255)						-- password text
);

--  PSA database version upgrade

replace into misc (param,val) values('version', '0600');

--  Fix skins table

ALTER TABLE Skins ADD UNIQUE (name);

--  Fix for data_bases table index

ALTER TABLE data_bases DROP INDEX name;
ALTER TABLE data_bases ADD UNIQUE (name,type);

--  Update domain templates

UPDATE TmplData SET value='0' WHERE element='stat_ttl' AND value='-1';

-- ---------------------------- IP Access restrivtion -----------------------------
CREATE TABLE cp_access (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY		-- IP Access record id, autogenerated
,	type	ENUM('allow', 'deny') NOT NULL DEFAULT 'allow'	-- cp acces table type
,	netaddr	CHAR(15) NOT NULL						-- IP Address
,	netmask	CHAR(15) NOT NULL						-- IP Netmask
);
