-- ----------------------------- Domains section -----------------------------------

-- subdomains table
CREATE TABLE subdomains (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY			-- wu ID
,	dom_id		INT UNSIGNED NOT NULL REFERENCES domains (id)	-- domain ID
,	name			VARCHAR(255) NOT NULL						-- domain name
,	UNIQUE (dom_id, name)
,	sys_user_type	ENUM('main','native') NOT NULL DEFAULT 'main'	-- system user type
,	sys_user_id	INT UNSIGNED NOT NULL REFERENCES sys_users (id)	-- sys_user ID
,	ssi			ENUM('false','true') NOT NULL DEFAULT 'false'	-- Server Side Includes support
,	php			ENUM('false','true') NOT NULL DEFAULT 'false'	-- PHP support
,	cgi			ENUM('false','true') NOT NULL DEFAULT 'false'	-- CGI support
,	perl		ENUM('false','true') NOT NULL DEFAULT 'false'	-- PERL support
,	python		ENUM('false','true') NOT NULL DEFAULT 'false'	-- PYTHON support
,	coldfusion	ENUM('false','true') NOT NULL DEFAULT 'false'	-- ColdFusion support
,	asp			ENUM('false','true') NOT NULL DEFAULT 'false'	-- ASP support
,	asp_dot_net	ENUM('false','true') NOT NULL DEFAULT 'false'	-- ASP.NET support
) TYPE=INNODB;

-- domains disk usage information
ALTER TABLE disk_usage
	ADD COLUMN	subdomains	BIGINT UNSIGNED DEFAULT 0 AFTER httpsdocs	-- space used by subdomains
,	ADD COLUMN	web_users	BIGINT UNSIGNED DEFAULT 0 AFTER subdomains	-- space used by web users
;

-- domains table correction
ALTER TABLE domains
	MODIFY cl_id		INT UNSIGNED NOT NULL DEFAULT 0		-- client ID - domain owner
,	MODIFY cert_rep_id	INT UNSIGNED DEFAULT 0				-- certificate repository ID
;

-- ---------------------------- Action Log section -----------------------------

-- List of logable actions: name (for php), descr (human-readable), enabled (bool)
replace into actions(name, descr, enabled) values ('subdomain_create', 'Create Subdomain', 'true');
replace into actions(name, descr, enabled) values ('subdomain_update', 'Update Subdomain', 'true');
replace into actions(name, descr, enabled) values ('subdomain_delete', 'Delete Subdomain', 'true');
replace into actions(name, descr, enabled) values ('siteapp_install', 'Install Site Application', 'true');
replace into actions(name, descr, enabled) values ('siteapp_reconfigure', 'Reconfigure Site Application', 'true');
replace into actions(name, descr, enabled) values ('siteapp_unchain', 'Unchain site application', 'true');
replace into actions(name, descr, enabled) values ('siteapp_uninstall', 'Uninstall site application', 'true');
replace into actions(name, descr, enabled) values ('siteapppkg_install', 'Install Site Application Package', 'true');
replace into actions(name, descr, enabled) values ('siteapppkg_update', 'Update of Site Application Package', 'true');
replace into actions(name, descr, enabled) values ('siteapppkg_uninstall', 'Uninstall Site Application Package', 'true');

-- ----------------------- Components versions section -------------------------

-- components list for statistic page
CREATE TABLE Components (
	name	varchar(255) PRIMARY KEY
,	version	varchar(255)
) TYPE=INNODB;

-- -------------------------------- Hosting section (ASP.NET) -------------------------------

-- domain
ALTER TABLE hosting
	ADD COLUMN asp_dot_net ENUM('false','true') NOT NULL DEFAULT 'false' AFTER asp
,	ADD COLUMN coldfusion	ENUM('false','true') NOT NULL DEFAULT 'false' AFTER python	-- ColdFusion support
,	ADD COLUMN at_domains	ENUM('false','true') NOT NULL DEFAULT 'false' AFTER webstat	-- @-domains support
,	DROP COLUMN rootchain
;

-- webuser
ALTER TABLE web_users
	ADD COLUMN asp_dot_net ENUM('false','true') NOT NULL DEFAULT 'false' AFTER asp;

-- ---------------------------- anonymous ftp section --------------------------

ALTER TABLE anon_ftp
	ADD COLUMN incoming_readable ENUM('false','true') NOT NULL DEFAULT 'false' AFTER incoming	-- Is incoming directory readable
,	ADD COLUMN incoming_subdirs ENUM('false','true') NOT NULL DEFAULT 'false' AFTER incoming_readable -- Enable create subdirs in incoming
;

-- -------------------------------- Mail section -------------------------------

-- badmailfrom table
CREATE TABLE badmailfrom (
	id		INT UNSIGNED AUTO_INCREMENT PRIMARY KEY		-- ID
,	domain	VARCHAR(255) NOT NULL						-- domain name
,		UNIQUE (domain)
) TYPE=INNODB;

-- ------------------------------ Sessions section -----------------------------

ALTER TABLE sessions
	ADD COLUMN type	SMALLINT UNSIGNED NOT NULL AFTER sess_id	-- session type (IS_ADMIN, IS_RESELLER, IS_CLIENT...)
,	ADD COLUMN ip_address char(15) NOT NULL AFTER login			-- IP Address number in a dot notation
,	ADD COLUMN login_time  DATETIME AFTER ip_address
;

-- ---------------------------- domain reports -----------------------------
CREATE TABLE domain_reports (
  id				int unsigned NOT NULL auto_increment PRIMARY KEY,
  user_id			int unsigned NOT NULL default '0',
  user_type 		int unsigned NOT NULL default '0',
  name 				varchar(255) default NULL,
  section_general	enum('none','summary','full') NOT NULL default 'summary',
  section_hosting	enum('none','summary','full') NOT NULL default 'summary',
  section_webusers	enum('none','summary','full') NOT NULL default 'summary',
  section_mailnames	enum('none','summary','full') NOT NULL default 'summary',
  section_maillists	enum('none','summary','full') NOT NULL default 'summary',
  section_databases	enum('none','summary','full') NOT NULL default 'summary',
  section_tomcat	enum('none','summary','full') NOT NULL default 'summary',
  section_siteapps	enum('none','summary','full') NOT NULL default 'summary',
  is_default		enum('false','true') NOT NULL default 'false'
) TYPE=INNODB;

-- ---------------------------- client reports -----------------------------
CREATE TABLE client_reports (
  id				int(10) unsigned NOT NULL auto_increment PRIMARY KEY,
  user_id			int(10) unsigned NOT NULL default '0',
  user_type			int(10) unsigned NOT NULL default '0',
  name				char(255) default NULL,
  section_general	enum('none','summary','full') NOT NULL default 'summary',
  section_domains	enum('none','summary','full') NOT NULL default 'summary',
  is_default		enum('false','true') NOT NULL default 'false'
) TYPE=INNODB;

-- ---------------------- custom buttons------------------------------------
ALTER table custom_buttons ADD cl_id INT UNSIGNED DEFAULT '0' AFTER id;
ALTER table custom_buttons ADD options INT UNSIGNED DEFAULT '0' AFTER conhelp;

-- ------------------------ CGI applications repository --------------------

CREATE TABLE SiteAppPackages (
	id				INT UNSIGNED AUTO_INCREMENT PRIMARY KEY	-- application id
,	name			VARCHAR(255) BINARY NOT NULL					-- short name
,	version			VARCHAR(30) BINARY NOT NULL							-- version
,	`release`			VARCHAR(30) BINARY NOT NULL							-- release
,	description		VARCHAR(255)							-- description
,	access_level	INT UNSIGNED NOT NULL DEFAULT 1			-- necessary access level to install application
,	integrated		ENUM('false', 'true') NOT NULL DEFAULT 'false'  -- integrated in Plesk
) TYPE=INNODB;

CREATE TABLE SiteApps (
	id			INT UNSIGNED AUTO_INCREMENT PRIMARY KEY	-- installed application id
,	dom_id		INT UNSIGNED NOT NULL REFERENCES domains (id) -- reference to domain, where application is installed
,	dom_type ENUM('domain', 'subdomain') NOT NULL DEFAULT 'domain' -- where application is installed
,	install_prefix	VARCHAR(255) BINARY NOT NULL				 -- directory of application inside vhost
,	htdocs_directory ENUM('httpdocs', 'httpsdocs') NOT NULL DEFAULT 'httpdocs' -- which documents tree is used for application
,	app_id		INT UNSIGNED NOT NULL REFERENCES AppsRepository (id) -- reference to application in repository
,	params_id	INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES Parameters (id) -- object id with values of parameters, that are used for installed application
) TYPE=INNODB;

CREATE TABLE SiteAppResources (
	app_id		INT UNSIGNED NOT NULL REFERENCES AppsInstalled (id)	-- reference to application
,	type		ENUM('database', 'dbuser') NOT NULL DEFAULT 'database'		-- name of type
,	res_id		INT UNSIGNED NOT NULL DEFAULT 0						-- reference to resource
,	UNIQUE(type, res_id)
) TYPE=INNODB;

-- ----------------------------- SpamAssassin section --------------------

CREATE TABLE sa_conf (
	mailname		VARCHAR(128) PRIMARY KEY							-- full mailname in form foo@bar.com (empty string in case of server)
,	flt_enabled		ENUM('user','serv', 'both') NOT NULL DEFAULT 'both'	-- which of filters enabled (for empty mailname, i.e. server, means enabledness of all users' filters)
,	rw_subject_tag	VARCHAR(255) NOT NULL DEFAULT '*****SPAM*****'		-- rewrite subject tag (if spam was detected)
,	hits_required	TINYINT(3) NOT NULL DEFAULT '7'						-- hits required for mark email as probably spam
,	spam_action			ENUM('delete','save') NOT NULL DEFAULT 'save'		-- what to do if spam recognized
) TYPE=INNODB;

CREATE TABLE sa_list (
	mailname	VARCHAR(128) REFERENCES sa_conf(mailname)	-- full mailname in form foo@bar.com (empty string in case of server)
,	pattern		VARCHAR(255) NOT NULL						-- shell-like mailsender pattern (From)
,	color		ENUM('black','white', 'gray') NULL			-- pattern color: black for blacklist, white for whitelist, gray for ignorelist,
															-- null means neither lists (and don't look for the pattern into common sa_list,
															-- common is marked with an empty mailname)
,	UNIQUE (mailname, pattern, color)
) TYPE=INNODB;

-- ---------------------- end of SpamAssassin section --------------------

-- ----------------------------- TTS section --------------------

CREATE TABLE tts_queues(
	id				INT UNSIGNED AUTO_INCREMENT PRIMARY KEY			-- queue ID, autogenerated
,	enabled			enum('false','true') NOT NULL default 'false'   -- Is enabled for using
,	title			VARCHAR(255) 									-- queue title
) TYPE=INNODB;

CREATE TABLE tts_priorities(
	id				INT UNSIGNED AUTO_INCREMENT PRIMARY KEY			-- priority ID, autogenerated
,	enabled			enum('false','true') NOT NULL default 'false'   -- Is enabled for using
,	title			VARCHAR(255) 									-- queue title
) TYPE=INNODB;

CREATE TABLE tts_cats(
	id				INT UNSIGNED AUTO_INCREMENT PRIMARY KEY			-- category ID, autogenerated
,	enabled			enum('false','true') NOT NULL default 'false'   -- Is enabled for using
,	title			VARCHAR(255) 									-- queue title
) TYPE=INNODB;

CREATE TABLE tts_conf (
	id					INT UNSIGNED PRIMARY KEY                            -- ID, setuped manualy
,	enabled				enum('false','true') NOT NULL default 'false'       -- Is enabled for clients
,	can_post			enum('false','true') NOT NULL default 'false'       -- Allow clients to post tickets
,	def_qu_id			INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES tts_queues (id)		-- Default Queue object ID
,	def_pr_id			INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES tts_priorites (id)	-- Default Priority object ID
,	def_cat_id			INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES tts_categories (id)	-- Default Category object ID
,	mg_enabled			enum('false','true') NOT NULL default 'false'       -- Is mailgate enabled
,	email_address       VARCHAR(255)										-- Send notifications from email
,	email_name          VARCHAR(255)										-- Send notifications from name
,	pop_server			VARCHAR(255)										-- POP3 server name for incoming tickets
,	pop_username		VARCHAR(255)										-- POP3 login
,	pop_password		VARCHAR(255)                                        -- POP3 password
,	query_period		INT UNSIGNED										-- Period for querying mail (in minutes)
,	start_ticket_subj	VARCHAR(255)										-- Ticket's subject must start with this prefix (for anti-spam)
) TYPE=INNODB;

CREATE TABLE tts_tickets (
	id 				INT UNSIGNED AUTO_INCREMENT PRIMARY KEY	 						-- Ticket ID, autogenerated
,	reporter_id		INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES tts_reporters (id)	-- Reporter ID
,	cat_id			INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES tts_cats(id)			-- Categories Object ID
,	qu_id			INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES tts_queues(id)		-- Queues Object ID
,	pr_id			INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES tts_priorites(id)	-- Priorites Object ID
,	status			BIGINT UNSIGNED NOT NULL DEFAULT 0								-- Ticket status bitmask
,	subject			VARCHAR(255)													-- Ticket subject
,	modified_date	DATETIME														-- Modified date
) TYPE=INNODB;

CREATE TABLE tts_reporters (
	id				INT UNSIGNED AUTO_INCREMENT PRIMARY KEY							-- Reporter ID, autogenerated
,	ref_reporter_id	INT UNSIGNED NOT NULL DEFAULT 0									-- Reference to clients(id), domain
,	type			enum('client', 'domain', 'mailname', 'email', 'admin') NOT NULL DEFAULT 'email' -- Ticket reporter ty
,	email			VARCHAR(255)													-- Mail-Client Email
,	name			VARCHAR(255)													-- Mail-Client Name
);

CREATE TABLE tts_ticket_events (
	id 				INT UNSIGNED AUTO_INCREMENT PRIMARY KEY	 						-- Event ID, autogenerated
,	ticket_id		INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES tts_tickets (id)		-- Tickets Object ID
,	event_type		BIGINT UNSIGNED NOT NULL DEFAULT 0								-- Event Type
,	public			enum('false','true') NOT NULL default 'false'       			-- Is visible for customers
,	reporter_id		INT UNSIGNED NOT NULL DEFAULT 0 REFERENCES tts_reporters (id)	-- Event Reporter ID
,	created_date	DATETIME														-- Creation date
,	descr			TEXT NOT NULL													-- Event Description
) TYPE=INNODB;

CREATE TABLE tts_slave_info(
	server_id		INT UNSIGNED NOT NULL DEFAULT 0 PRIMARY KEY REFERENCES slaves (id)		-- Slaves ID
,	enabled			enum('false','true') NOT NULL default 'false'       -- Is enabled for clients
,	can_post		enum('false','true') NOT NULL default 'false'       -- Allow clients to post tickets
,	cnt_new			BIGINT UNSIGNED NOT NULL DEFAULT 0					-- New tickets
,	cnt_opened		BIGINT UNSIGNED NOT NULL DEFAULT 0					-- Opened tickets
,	cnt_reopened	BIGINT UNSIGNED NOT NULL DEFAULT 0					-- Repened tickets
,	cnt_closed		BIGINT UNSIGNED NOT NULL DEFAULT 0					-- Closed tickets
,	updated_date	DATETIME											-- Latest Updated Date
) TYPE=INNODB;

delete from tts_queues;
replace into tts_queues (id, enabled, title) values(1, 'true', '1');
replace into tts_queues (id, enabled, title) values(2, 'true', '2');
replace into tts_queues (id, enabled, title) values(3, 'true', '3');

delete from tts_priorities;
replace into tts_priorities (id, enabled, title) values(1, 'true', '1-High');
replace into tts_priorities (id, enabled, title) values(2, 'true', '2-Normal');
replace into tts_priorities (id, enabled, title) values(3, 'true', '3-Low');

delete from tts_cats;
replace into tts_cats (id, enabled, title) values(1, 'true', 'General');
replace into tts_cats (id, enabled, title) values(2, 'true', 'Hosting');
replace into tts_cats (id, enabled, title) values(3, 'true', 'Mail');
replace into tts_cats (id, enabled, title) values(4, 'true', 'DNS');
replace into tts_cats (id, enabled, title) values(5, 'true', 'Database');
replace into tts_cats (id, enabled, title) values(6, 'true', 'FTP');
replace into tts_cats (id, enabled, title) values(7, 'true', 'Site Applications');
replace into tts_cats (id, enabled, title) values(8, 'true', 'Tomcat');
replace into tts_cats (id, enabled, title) values(9, 'true', 'Mailing Lists');

delete from tts_conf;
replace into tts_conf (id, enabled, can_post, def_qu_id, def_pr_id, def_cat_id) values(1, 'true', 'true', 1, 2, 1);

UPDATE Notes SET text="A new client account has been created in Plesk.\nClient's contact name: <client_contact_name>\nClient's login: <client_login>\nClient's password: <password>\nPlesk entry point: https://<hostname>:8443" WHERE id=1 AND text="This is a notification of new client creation from Plesk.\nClient contact name: <client_contact_name>\nClient login: <client_login>\nClient password: <password>\nPlesk entry point: https://<hostname>:8443";
UPDATE Notes SET text="A new domain has been created in Plesk.\nDomain name: <domain>\nClient owner: <client_contact_name>\nDomain IP: <ip>" WHERE id=2 AND text="This is a notification of new domain creation from Plesk.\nDomain name: <domain_name>\nClient owner: <client_contact_name>\nDomain IP: <ip>";
UPDATE Notes SET text="The validity period of the client account '<client_login>' (Contact Name '<client_contact_name>') has expired.\nThe client account and all related domains were deactivated on <expiration_date>." WHERE id=4 AND text="The period of validity of the client '<client_login>' (Contact Name '<client_contact_name>') has expired.\nThe client and all of his/her domains were deactivated on <expiration_date>.";
UPDATE Notes SET text="The validity period of the client account '<client_login>' (Contact Name '<client_contact_name>') is about to expire.\nThe client account and all of related domains will be deactivated by <expiration_date> unless the validity period is prolonged." WHERE id=5 AND text="The period of validity of the client '<client_login>' (Contact Name '<client_contact_name>') is about to expire.\nThe client and all of his/her domains will be deactivated by <expiration_date> unless the period of validity is prolonged.";
UPDATE Notes SET text="The validity period of the '<domain_name>' domain owned by client '<client_login>' (Contact name '<client_contact_name>') has expired.\nThe domain and all of its services were deactivated on <expiration_date>." WHERE id=6 AND text="The period of validity of the domain '<domain_name>' owned by client '<client_login>' (Contact name '<client_contact_name>') has expired.\nThe domain and all of its services were deactivated on <expiration_date>.";
UPDATE Notes SET text="The validity period of the '<domain_name>' domain owned by client '<client_login>' (Contact name '<client_contact_name>') is about to expire.\nThe domain and all of its services will be deactivated by <expiration_date> unless the validity period is prolonged." WHERE id=7 AND text="The period of validity of the domain '<domain_name>' owned by client '<client_login>' (Contact name '<client_contact_name>') is about to expire.\nThe domain and all of its services will be deactivated by <expiration_date> unless the period of validity is prolonged.";

replace into Notes (id, text) values (8, "A new ticket <ticket_id> has been submitted to the Help Desk.\nPlease retain the ticket identification number for future references. Below is a copy of the ticket.\n-----\n<ticket_comment>\n-----");
replace into Notes (id, text) values (9, "Ticket <ticket_id> has been commented.\nBelow is the comment.\n-----\n<ticket_comment>\n-----");
replace into Notes (id, text) values (10,"Ticket <ticket_id> has been closed.\nBelow is the comment.\n-----\n<ticket_comment>\n-----");
replace into Notes (id, text) values (11,"Ticket <ticket_id> has been reopened.\nBelow is the comment.\n-----\n<ticket_comment>\n-----");

replace into Notifications (id, note_id, subj, send2admin, send2client) values (8, 8, 'Help Desk event notification: Ticket has been created.', 1, 1);
replace into Notifications (id, note_id, subj, send2admin, send2client) values (9, 9, 'Help Desk event notification: Ticket has been commented.', 1, 1);
replace into Notifications (id, note_id, subj, send2admin, send2client) values (10,10,'Help Desk event notification: Ticket has been closed.', 1, 1);
replace into Notifications (id, note_id, subj, send2admin, send2client) values (11,11,'Help Desk event notification: Ticket has been reopened.', 1, 1);

-- ----------------------------- /TTS section --------------------

alter table data_bases modify type ENUM('mysql', 'postgresql', 'mssql') NOT NULL DEFAULT 'mysql'; -- note, that postgres type is changed to postgresql
update data_bases set type='postgresql' where type='';

alter table IP_Addresses modify iface VARCHAR(255) NOT NULL;
alter table clients modify locale VARCHAR(3) NOT NULL DEFAULT 'en';
alter table sessions modify login_time DATETIME NOT NULL;
alter table sessions change exp_date click_time TIMESTAMP NOT NULL;
alter table certificates
	drop column time_stamp
,	modify csr BLOB NOT NULL DEFAULT ''
,	modify pvt_key BLOB NOT NULL DEFAULT ''
,	modify cert BLOB NOT NULL DEFAULT ''
,	modify cert_file VARCHAR(255) NOT NULL DEFAULT ''
,	modify ca_cert BLOB NOT NULL DEFAULT ''
,	modify ca_file VARCHAR(255) NOT NULL DEFAULT ''
,	modify name VARCHAR(255) NOT NULL DEFAULT ''
;
alter table disk_usage add column domaindumps BIGINT UNSIGNED DEFAULT 0 after maillists;

--  PSA database version upgrade

replace into misc (param,val) values('version', '0700');

-- -------------------------- and set new default skin ------------------

UPDATE misc SET val='1' WHERE param='def_skin_id';
UPDATE misc SET val='1' WHERE param='admin_skin_id';
UPDATE cl_param SET val='1' WHERE param='skin_id';
UPDATE dom_param SET val='1' WHERE param='user_skin_id';
UPDATE mn_param SET val='1' WHERE param='skin_id';

-- ----------------- rename postgresql parameters in misc table ---------

UPDATE misc SET param='postgresql_admin_login' WHERE param='pg_admin_login';
UPDATE misc SET param='postgresql_admin_passwd' WHERE param='pg_admin_pass';

-- ----------------- allow multiple logins by default (bug #24976) ------
REPLACE INTO misc (param,val) VALUES('multiply_login', 'true');													-- administrator
REPLACE INTO cl_param  SELECT id     AS cl_id,  'multiply_login' AS param, 'true' AS val FROM clients;			-- clients
REPLACE INTO dom_param SELECT dom_id AS dom_id, 'multiply_login' AS param, 'true' AS val FROM dom_level_usrs;	-- domain-level users
REPLACE INTO mn_param  SELECT id     AS mn_id,  'multiply_login' AS param, 'true' AS val FROM mail;				-- mail users

-- --------------- update master feature parameters ---------------------

replace into misc (param,val) values('slave_services_list_with_non_obligatory_status_checking', 'tomcat,postgresql,coldfusion,spamd');
replace into misc (param,val) values('master_services_list_for_obligatory_status_checking', 'pop3,smtp,http,ftp,dns');

-- ------------- unification of measurement units -----------------------

UPDATE sys_users SET quota = quota * 1048576 WHERE quota > 0;
UPDATE mail SET mbox_quota = mbox_quota * 1024 WHERE mbox_quota > 0;
UPDATE Limits SET value = value * 1024 WHERE limit_name='mbox_quota' AND value > 0;
UPDATE TmplData SET value = value * 1024 WHERE element='mbox_quota' and value > 0;
UPDATE TmplData SET value = value * 1048576 WHERE element='quota' and value > 0;
UPDATE log_rotation SET period = period * 1024 WHERE period_type='by_size';

-- ----------------- switching to innodb type ---------------------------

ALTER TABLE accounts TYPE=INNODB;
ALTER TABLE clients TYPE=INNODB;
ALTER TABLE cl_param TYPE=INNODB;
ALTER TABLE dom_level_usrs TYPE=INNODB;
ALTER TABLE domains TYPE=INNODB;
ALTER TABLE dom_param TYPE=INNODB;
ALTER TABLE DomainServices TYPE=INNODB;
ALTER TABLE WebApps TYPE=INNODB;
ALTER TABLE disk_usage TYPE=INNODB;
ALTER TABLE dns_recs TYPE=INNODB;
ALTER TABLE dns_masters TYPE=INNODB;
ALTER TABLE dns_recs_t TYPE=INNODB;
ALTER TABLE hosting TYPE=INNODB;
ALTER TABLE web_users TYPE=INNODB;
ALTER TABLE protected_dirs TYPE=INNODB;
ALTER TABLE pd_users TYPE=INNODB;
ALTER TABLE log_rotation TYPE=INNODB;
ALTER TABLE forwarding TYPE=INNODB;
ALTER TABLE anon_ftp TYPE=INNODB;
ALTER TABLE DomainsTraffic TYPE=INNODB;
ALTER TABLE ClientsTraffic TYPE=INNODB;
ALTER TABLE certificates TYPE=INNODB;
ALTER TABLE mail TYPE=INNODB;
ALTER TABLE mail_aliases TYPE=INNODB;
ALTER TABLE mail_redir TYPE=INNODB;
ALTER TABLE mail_resp TYPE=INNODB;
ALTER TABLE resp_freq TYPE=INNODB;
ALTER TABLE resp_attach TYPE=INNODB;
ALTER TABLE resp_forward TYPE=INNODB;
ALTER TABLE smtp_poplocks TYPE=INNODB;
ALTER TABLE mn_param TYPE=INNODB;
ALTER TABLE Notifications TYPE=INNODB;
ALTER TABLE Notes TYPE=INNODB;
ALTER TABLE locales TYPE=INNODB;
ALTER TABLE lockout TYPE=INNODB;
ALTER TABLE Logos TYPE=INNODB;
ALTER TABLE data_bases TYPE=INNODB;
ALTER TABLE db_users TYPE=INNODB;
ALTER TABLE Cards TYPE=INNODB;
ALTER TABLE sys_users TYPE=INNODB;
ALTER TABLE Permissions TYPE=INNODB;
ALTER TABLE Parameters TYPE=INNODB;
ALTER TABLE Limits TYPE=INNODB;
ALTER TABLE Templates TYPE=INNODB;
ALTER TABLE TmplData TYPE=INNODB;
ALTER TABLE Repository TYPE=INNODB;
ALTER TABLE Skins TYPE=INNODB;
ALTER TABLE slaves TYPE=INNODB;
ALTER TABLE slave_params TYPE=INNODB;
ALTER TABLE sessions TYPE=INNODB;
ALTER TABLE misc TYPE=INNODB;
ALTER TABLE mailer_queries TYPE=INNODB;
ALTER TABLE custom_buttons TYPE=INNODB;
ALTER TABLE upgrade_history TYPE=INNODB;
ALTER TABLE ListsParams TYPE=INNODB;
ALTER TABLE IP_Addresses TYPE=INNODB;
ALTER TABLE actions TYPE=INNODB;
ALTER TABLE log_actions TYPE=INNODB;
ALTER TABLE log_components TYPE=INNODB;
ALTER TABLE event_handlers TYPE=INNODB;
ALTER TABLE MailLists TYPE=INNODB;
ALTER TABLE cp_access TYPE=INNODB;
