-- ----------------------------- clients --------------------
ALTER TABLE clients ADD COLUMN sapp_pool_id INT UNSIGNED NOT NULL DEFAULT 0;
ALTER TABLE clients ADD INDEX (sapp_pool_id);
-- ----------------------------- /clients --------------------

-- ----------------------------- pd_users --------------------
ALTER TABLE pd_users ADD COLUMN account_id INT UNSIGNED NOT NULL AFTER login;
ALTER TABLE pd_users ADD INDEX (account_id);

-- move data
ALTER TABLE accounts ADD COLUMN pd_users_id int unsigned;
INSERT INTO accounts (type, password, pd_users_id)
	SELECT 'plain', passwd, id from pd_users;
SELECT p.id, p.login, p.pd_id, a.id as account_id
	INTO OUTFILE '/tmp/pd_users.dmp'
	FROM pd_users as p, accounts as a
		WHERE a.pd_users_id = p.id;
LOAD DATA INFILE '/tmp/pd_users.dmp'
	REPLACE INTO TABLE pd_users(id, login, pd_id, account_id);
ALTER TABLE accounts DROP COLUMN pd_users_id;

-- now finish structure change
ALTER TABLE pd_users  DROP COLUMN passwd;
-- ----------------------------- /pd_users --------------------

-- ----------------------------- protected_dirs --------------------
UPDATE protected_dirs SET path = CONCAT('plesk-stat/', path) WHERE path IN ('webstat', 'ftpstat');
-- ----------------------------- /protected_dirs --------------------

-- ----------------------------- anon_ftp --------------------
ALTER TABLE anon_ftp ADD COLUMN display_login	ENUM('false','true') NOT NULL DEFAULT 'false';
ALTER TABLE anon_ftp ADD COLUMN login_text	TEXT;
-- ----------------------------- /anon_ftp --------------------

-- -------------------------------------- CUSTOM BUTTONS ---------------------------------

ALTER TABLE custom_buttons ADD COLUMN sort_key INT UNSIGNED NOT NULL DEFAULT 100  AFTER id;
ALTER TABLE custom_buttons ADD COLUMN level INT UNSIGNED NOT NULL DEFAULT 0  AFTER sort_key;
ALTER TABLE custom_buttons CHANGE cl_id level_id INT UNSIGNED NOT NULL DEFAULT 0;
ALTER TABLE custom_buttons ADD COLUMN place enum('domain', 'client', 'navigation') NOT NULL DEFAULT 'domain'  AFTER level_id;
ALTER TABLE custom_buttons ADD COLUMN file VARCHAR(245) NOT NULL;

UPDATE custom_buttons SET level = "1" WHERE level_id = "0";
UPDATE custom_buttons SET level = "4" WHERE level_id != "0";
UPDATE custom_buttons SET place = "domain";
UPDATE custom_buttons SET options = options | 128;

-- -------------------------------------- SITE APPS ---------------------------------

-- -------------------------- reports ----------------------------
CREATE TABLE report (
	id			INT(10) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY
,	type			INT(10) UNSIGNED NOT NULL DEFAULT '0'
,	user_id			INT(10) UNSIGNED NOT NULL DEFAULT '0'
,	user_type		INT(10) UNSIGNED NOT NULL DEFAULT '0'
,	name			VARCHAR(255) DEFAULT NULL
,	is_default		ENUM('false','true') NOT NULL DEFAULT 'false'
,	old_report_type	ENUM('domain','client') NOT NULL
,	old_report_id	INT(10) UNSIGNED NOT NULL
,   UNIQUE(old_report_type, old_report_id)
) TYPE=INNODB;

CREATE TABLE report_section (
	id			INT(10) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY
,	report_id		INT(10) UNSIGNED NOT NULL DEFAULT '0'
,	name			VARCHAR(255) NOT NULL DEFAULT ''
,	type			ENUM('none','summary','full') NOT NULL DEFAULT 'summary'
) TYPE=INNODB;

CREATE TABLE report_auto (
	id			INT(10) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY
,	report_id		INT(10) UNSIGNED NOT NULL DEFAULT '0'
,	auto			INT(10) UNSIGNED NOT NULL DEFAULT '0'
,	last			DATETIME DEFAULT NULL
,	recipient		INT(10) UNSIGNED NOT NULL DEFAULT '0'
,	email			VARCHAR(254) NOT NULL DEFAULT ''
,	client			INT(10) UNSIGNED DEFAULT '0'
,	domain			INT(10) UNSIGNED DEFAULT '0'
) TYPE=INNODB;

INSERT INTO report(type, user_id, user_type, name, is_default, old_report_type, old_report_id)
	SELECT 16 AS type, user_id, user_type<<1, name, is_default, 'domain', id
	FROM domain_reports;

INSERT INTO report_section(report_id, name, type)
	SELECT r.id AS report_id, 'general' AS name, pr.section_general AS type
	FROM report AS r, domain_reports AS pr
	WHERE r.old_report_id = pr.id and r.old_report_type = 'domain' and pr.section_general <> 'none';

INSERT INTO report_section(report_id, name, type)
	SELECT r.id AS report_id, 'hosting' AS name, pr.section_hosting AS type
	FROM report AS r, domain_reports AS pr
	WHERE r.old_report_id = pr.id and r.old_report_type = 'domain' and pr.section_hosting <> 'none';

INSERT INTO report_section(report_id, name, type)
	SELECT r.id AS report_id, 'webusers' AS name, pr.section_webusers AS type
	FROM report AS r, domain_reports AS pr
	WHERE r.old_report_id = pr.id and r.old_report_type = 'domain' and pr.section_webusers <> 'none';

INSERT INTO report_section(report_id, name, type)
	SELECT r.id AS report_id, 'mailnames' AS name, pr.section_mailnames AS type
	FROM report AS r, domain_reports AS pr
	WHERE r.old_report_id = pr.id and r.old_report_type = 'domain' and pr.section_mailnames <> 'none';

INSERT INTO report_section(report_id, name, type)
	SELECT r.id AS report_id, 'maillists' AS name, pr.section_maillists AS type
	FROM report AS r, domain_reports AS pr
	WHERE r.old_report_id = pr.id and r.old_report_type = 'domain' and pr.section_maillists <> 'none';

INSERT INTO report_section(report_id, name, type)
	SELECT r.id AS report_id, 'databases' AS name, pr.section_databases AS type
	FROM report AS r, domain_reports AS pr
	WHERE r.old_report_id = pr.id and r.old_report_type = 'domain' and pr.section_databases <> 'none';

INSERT INTO report_section(report_id, name, type)
	SELECT r.id AS report_id, 'tomcat' AS name, pr.section_tomcat AS type
	FROM report AS r, domain_reports AS pr
	WHERE r.old_report_id = pr.id and r.old_report_type = 'domain' and pr.section_tomcat <> 'none';

INSERT INTO report_section(report_id, name, type)
	SELECT r.id AS report_id, 'siteapps' AS name, pr.section_siteapps AS type
	FROM report AS r, domain_reports AS pr
	WHERE r.old_report_id = pr.id and r.old_report_type = 'domain' and pr.section_siteapps <> 'none';

INSERT INTO report(type, user_id, user_type, name, is_default, old_report_type, old_report_id)
	SELECT 8 AS type, user_id, user_type<<1, name, is_default, 'client', id
	FROM client_reports;

INSERT INTO report_section(report_id, name, type)
	SELECT r.id AS report_id, 'general' AS name, pr.section_general AS type
	FROM report AS r, client_reports AS pr
	WHERE r.old_report_id = pr.id and r.old_report_type = 'client' and pr.section_general <> 'none';

INSERT INTO report_section(report_id, name, type)
	SELECT r.id AS report_id, 'domains' AS name, pr.section_domains AS type
	FROM report AS r, client_reports AS pr
	WHERE r.old_report_id = pr.id and r.old_report_type = 'client' and pr.section_domains <> 'none';

ALTER TABLE report DROP old_report_type, DROP old_report_id;
DROP TABLE domain_reports;
DROP TABLE client_reports;
-- -------------------------- /reports ----------------------------

ALTER TABLE SiteAppResources MODIFY type ENUM('database', 'dbuser', 'custom_button') NOT NULL DEFAULT 'database';

-- -------------------------- Site Application Packages ------------------------
ALTER TABLE SiteAppPackages ADD UNIQUE (name, version, `release`);
ALTER TABLE SiteAppPackages MODIFY access_level INT UNSIGNED NOT NULL DEFAULT 0;
UPDATE SiteAppPackages SET access_level = "0";

--  PSA database version upgrade
replace into misc (param,val) values('version', '0710');

-- -------------------------------------- domains ---------------------------------
ALTER TABLE domains ADD COLUMN displayName VARCHAR(255) NOT NULL AFTER name;
UPDATE domains SET displayName = name;
ALTER TABLE domains ADD UNIQUE (displayName);
-- -------------------------------------- /domains ---------------------------------

-- -------------------------------------- subdomains ---------------------------------
ALTER TABLE subdomains ADD COLUMN displayName VARCHAR(255) NOT NULL AFTER name;
UPDATE subdomains SET displayName = name;
ALTER TABLE subdomains ADD UNIQUE displayName(dom_id, displayName);
-- -------------------------------------- /subdomains ---------------------------------

-- ----------------------------- KeyHistory --------------------
CREATE TABLE key_history (
	id				INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
,	plesk_key_id	VARCHAR(63)
,	name			VARCHAR(255) BINARY
,	filename		VARCHAR(255) BINARY
,	register_date	TIMESTAMP not NULL
,	update_disabled	enum('false', 'true') not null default 'false'
,	options			VARCHAR(255) BINARY
) TYPE=INNODB;
-- ----------------------------- /KeyHistory --------------------

-- ----------------------------- KeyHistoryParams ----------------------
CREATE TABLE key_history_params (
	key_id	INT UNSIGNED NOT NULL REFERENCES key_history(id)
,	param	VARCHAR(63) NOT NULL
,	val		VARCHAR(255)
,	PRIMARY KEY (key_id, param)
) TYPE=INNODB;
-- ----------------------------- /KeyHistoryParams ----------------------

-- ----------------------------- event_handlers ----------------------
UPDATE event_handlers SET priority = ((priority + 255)*100)/510;
-- ----------------------------- /event_handlers ----------------------

-- ----------------------------------- action logging --------------------------
replace into actions(name, descr, enabled) values ('service_stop', 'Stop Service', 'true');
replace into actions(name, descr, enabled) values ('service_start', 'Start Service', 'true');
replace into actions(name, descr, enabled) values ('service_restart', 'Restart Service', 'true');
replace into actions(name, descr, enabled) values ('admin_update', 'Update Administrator Information', 'true');
replace into actions(name, descr, enabled) values ('ip_address_create', 'Create IP Address', 'true');
replace into actions(name, descr, enabled) values ('ip_address_update', 'Update IP Address', 'true');
replace into actions(name, descr, enabled) values ('ip_address_delete', 'Delete IP Address', 'true');
replace into actions(name, descr, enabled) values ('domain_status_update', 'Update Domain Status', 'true');
replace into actions(name, descr, enabled) values ('domain_dns_update', 'Update Domain DNS Zone', 'true');
replace into actions(name, descr, enabled) values ('domain_limit_size_reached', 'Domain\'s disk space limit reached', 'true');
replace into actions(name, descr, enabled) values ('domain_limit_traffic_reached', 'Domain\'s traffic limit reached', 'true');
replace into actions(name, descr, enabled) values ('client_permissions_update', 'Update Client Permissions', 'true');
replace into actions(name, descr, enabled) values ('client_status_update', 'Update Client Status', 'true');
replace into actions(name, descr, enabled) values ('client_preferences_update', 'Update Client Preferences', 'true');
replace into actions(name, descr, enabled) values ('client_ip_pool_update', 'Update Client IP Pool', 'true');
replace into actions(name, descr, enabled) values ('client_limit_size_reached', 'Client\'s disk space limit reached', 'true');
replace into actions(name, descr, enabled) values ('client_limit_traffic_reached', 'Client\'s traffic limit reached', 'true');
replace into actions(name, descr, enabled) values ('forwarding_create', 'Create Standard/Frame forwarding', 'true');
replace into actions(name, descr, enabled) values ('forwarding_update', 'Update Standard/Frame forwarding', 'true');
replace into actions(name, descr, enabled) values ('forwarding_delete', 'Delete Standard/Frame forwarding', 'true');
replace into actions(name, descr, enabled) values ('license_update', 'License updated', 'true');
