-- -------------- dashboard preset
CREATE TABLE IF NOT EXISTS DashboardPreset (
	id						INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
								-- Dashboard preset ID, autogenerated
,	name					VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci DEFAULT ''
								-- Dashboard preset name
,	uri						VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL
								-- Dashboard preset root uri
,	type					ENUM('admin', 'client', 'domain', 'private', 'custom') NOT NULL DEFAULT 'private'
								-- Dashboard preset type
,							UNIQUE name_type (name, type)
) TYPE=InnoDB;

-- -------------- dashboard preset element
CREATE TABLE IF NOT EXISTS DashboardPresetConfig (
	id						INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
								-- Dashboard preset element ID, autogenerated
,	preset_id				INT UNSIGNED NOT NULL
								-- Dashboard preset id
,	ord						INT UNSIGNED NOT NULL
								-- Dashboard element order
,	uri						VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL
								-- Dashboard preset element uri
,	parent_id				INT UNSIGNED DEFAULT NULL
								-- Parent dashboard preset element ID
,	type					ENUM('none', 'button', 'custom-buttons', 'column', 'footer', 'indicator', 'left', 'right', 'section', 'shortcut', 'tab', 'tabs', 'quick-search', 'quick-search-option') NOT NULL DEFAULT 'none'
								-- Dashboard element type
,	title					VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci DEFAULT ''
								-- Dasboard element title
,	description				VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci DEFAULT ''
								-- Dashboard element description
,	enabled					ENUM('false', 'true') NOT NULL DEFAULT 'true'
								-- Dashboard element enabled
) TYPE=InnoDB;

-- Product news articles
CREATE TABLE IF NOT EXISTS NewsArticles (
    id                      INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
                                -- ID, autogenerated
,   class                   VARCHAR(255) CHARACTER SET ascii COLLATE ascii_bin NOT NULL
                                -- News class
,   data                    TEXT CHARACTER SET binary NULL
                                -- Class-specific data
) TYPE=InnoDB;

ALTER TABLE custom_buttons MODIFY place	ENUM('domain', 'client', 'admin', 'navigation') NOT NULL DEFAULT 'domain';

-- default presets configuration
REPLACE INTO DashboardPreset (id, uri, name, type) values (1, '/plesk/dashboard/', 'Default Administrator Desktop', 'admin');
REPLACE INTO misc (param, val) VALUES ('def_admin_preset_id', 1);
REPLACE INTO misc (param, val) VALUES ('admin_preset_id', 1);

REPLACE INTO DashboardPreset (id, uri, name, type) values (2, '/plesk/client@/dashboard/', 'Default Client Desktop', 'client');
REPLACE INTO misc (param, val) VALUES ('def_client_preset_id', 2);

REPLACE INTO DashboardPreset (id, uri, name, type) values (3, '/plesk/client@/domain@/dashboard/', 'Default Domain Owner Desktop', 'domain');
REPLACE INTO misc (param, val) VALUES ('def_domain_preset_id', 3);

-- Product news
INSERT INTO NewsArticles(class) VALUES ('WhatsNew800');

-- New skin
REPLACE INTO Skins (name, place) VALUES ('Plesk Reflection', 'plesk8.new');

-- Action log
replace into actions(name, descr, enabled) values ('dashboard_preset_create', 'Create Desktop Preset', 'true');
replace into actions(name, descr, enabled) values ('dashboard_preset_update', 'Update Desktop Preset', 'true');
replace into actions(name, descr, enabled) values ('dashboard_preset_delete', 'Delete Desktop Preset', 'true');
