
-- password secrets
CREATE TABLE IF NOT EXISTS password_secrets (
	secret					VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci PRIMARY KEY,
								-- secret key
	type					INT UNSIGNED NOT NULL,
								-- client type (IS_CLIENT, IS_DOMAIN_OWNER, IS_MAIL_USER)
	id						INT UNSIGNED NOT NULL,
								-- client ID
	created					DATETIME NOT NULL
								-- time of creation
) TYPE=InnoDB;

-- Control visibility customizations
CREATE TABLE IF NOT EXISTS itmpl (
	id						INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
								-- ID, autogenerated
,	name					VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci
								-- template name
) TYPE=InnoDB;

CREATE TABLE IF NOT EXISTS itmpl_data (
	itmpl_id					INT UNSIGNED NOT NULL
								-- interface template id
,	page					VARCHAR(127) CHARACTER SET ascii COLLATE ascii_bin NOT NULL
								-- page on which the control is
,	control					VARCHAR(127) CHARACTER SET ascii COLLATE ascii_bin NOT NULL
								-- control ID
,	state					ENUM('hide', 'adminOnly', 'show') NOT NULL DEFAULT 'hide'
								-- custom visibility
,	control_type			ENUM('button', 'formControl') NOT NULL DEFAULT 'button'
								-- type of control
,							PRIMARY KEY (itmpl_id, page, control)
) TYPE=InnoDB;

-- referrers to hide
CREATE TABLE IF NOT EXISTS webalizer_hidden_referrer (
	id						INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
								-- Record ID, autogenerated
,	dom_id					INT UNSIGNED NOT NULL
								-- Domain ID
,	referrer				VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
								-- Referrer value
) TYPE=InnoDB;

-- referrer groups
CREATE TABLE IF NOT EXISTS webalizer_group_referrer (
	id						INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
								-- Record ID, autogenerated
,	dom_id					INT UNSIGNED NOT NULL
								-- Domain ID
,	group_name				VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
								-- Referrer group name
,	referrer				VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
								-- Referrer value
) TYPE=InnoDB;

CREATE TABLE IF NOT EXISTS SiteAppFiles (
	instance_id				INT UNSIGNED NOT NULL
,	prefix					ENUM('htdocs', 'cgi-bin') NOT NULL
,	`file`					VARCHAR(255) CHARACTER SET binary NOT NULL
,		INDEX instance_id (instance_id)
) TYPE=InnoDB;

CREATE TABLE IF NOT EXISTS DatabaseServers (
	id						INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
								-- ID, autogenerated
,	host					VARCHAR(63) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
								-- Host name or IP address
,	port					INT UNSIGNED
								-- TCP port number
,	type					ENUM('mysql', 'postgresql') NOT NULL
								-- Database server type
,	admin_login				VARCHAR(255) CHARACTER SET binary NOT NULL
								-- Administrator login
,	admin_password			VARCHAR(255) CHARACTER SET binary NOT NULL
								-- Administrator password
,	last_error				ENUM('no_error', 'connection_failed',
							     'permission_denied', 'other_error',
							     'credentials_not_set') NOT NULL DEFAULT 'no_error'
								-- Last error
,							UNIQUE(host, port)
) TYPE=InnoDB;

-- dns_zone
CREATE TABLE IF NOT EXISTS dns_zone (
	id					INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
								-- ID
,	name					VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL
								-- zone ascii name
,	displayName				VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
								-- zone display name
,	status					INT UNSIGNED NOT NULL DEFAULT 0
								-- DNS zone status (STATUS_ACTIVE, STATUS_STATUS, STATUS_WITH_PARENT)
,	email					VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci
								-- admin email
,	type					ENUM('slave','master') NOT NULL DEFAULT 'master'
								-- DNS zone type
,	ttl						INT UNSIGNED NOT NULL DEFAULT 86400
								-- SOA TTL
,	ttl_unit				INT UNSIGNED NOT NULL DEFAULT 1
								-- SOA TTL UNITS
,	refresh					INT UNSIGNED NOT NULL DEFAULT 10800
								-- SOA REFRESH
,	refresh_unit			INT UNSIGNED NOT NULL DEFAULT 1
								-- SOA REFRESH UNITS
,	retry					INT UNSIGNED NOT NULL DEFAULT 3600
								-- SOA RETRY
,	retry_unit				INT UNSIGNED NOT NULL DEFAULT 1
								-- SOA RETRY UNITS
,	expire					INT UNSIGNED NOT NULL DEFAULT 604800
								-- SOA EXPIRE
,	expire_unit				INT UNSIGNED NOT NULL DEFAULT 1
								-- SOA EXPIRE UNITS
,	minimum					INT UNSIGNED NOT NULL DEFAULT 10800
								-- SOA MINIMUM
,	minimum_unit			INT UNSIGNED NOT NULL DEFAULT 1
								-- SOA MINIMUM UNITS
) TYPE=InnoDB;

-- domain aliases
CREATE TABLE IF NOT EXISTS domainaliases (
	id					INT UNSIGNED AUTO_INCREMENT PRIMARY KEY
								-- ID
,	dom_id					INT UNSIGNED NOT NULL
								-- domain ID
,	dns_zone_id				INT UNSIGNED NOT NULL
								-- DNS zone ID
,	status					INT UNSIGNED NOT NULL DEFAULT 0
								-- domain alias status (STATUS_ACTIVE, STATUS_STATU, STATUS_WITH_PARENT)
,	name					VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL
								-- domain alias name
,	displayName				VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
								-- original domain alias name as was created
,	mail					ENUM('false', 'true') NOT NULL DEFAULT 'false'
								-- enable mail aliasing
,	web						ENUM('false', 'true') NOT NULL DEFAULT 'false'
								-- enable web aliasing
,							UNIQUE (name)
,							INDEX (dom_id)
								-- REFERENCES domains (id)
,							INDEX (dns_zone_id)
								-- REFERENCES dns_zone (id)
) TYPE=InnoDB;

--

ALTER TABLE clients MODIFY limits_id INT UNSIGNED;
ALTER TABLE clients MODIFY params_id INT UNSIGNED;
ALTER TABLE clients MODIFY perm_id INT UNSIGNED;
ALTER TABLE clients MODIFY pool_id INT UNSIGNED;
ALTER TABLE clients MODIFY logo_id INT UNSIGNED;
ALTER TABLE clients MODIFY tmpl_id INT UNSIGNED;
ALTER TABLE clients MODIFY sapp_pool_id INT UNSIGNED;

ALTER TABLE dom_level_usrs MODIFY account_id INT UNSIGNED;

ALTER TABLE domains MODIFY cert_rep_id INT UNSIGNED;
ALTER TABLE domains MODIFY limits_id INT UNSIGNED;
ALTER TABLE domains MODIFY params_id INT UNSIGNED;

UPDATE DomainServices SET status = 0 WHERE status IS NULL;
ALTER TABLE DomainServices MODIFY status BIGINT UNSIGNED NOT NULL DEFAULT 0;
ALTER TABLE DomainServices MODIFY parameters_id INT UNSIGNED;

ALTER TABLE IP_Addresses MODIFY default_domain_id INT UNSIGNED;

ALTER TABLE dns_recs
	MODIFY `type` ENUM('NS','A','CNAME','MX','PTR','TXT', 'master', 'none') NOT NULL DEFAULT 'A';

UPDATE WebApps SET status = 0 WHERE status IS NULL;
ALTER TABLE WebApps MODIFY status INT NOT NULL DEFAULT 0;

ALTER TABLE sessions MODIFY login VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL;

ALTER TABLE lockout MODIFY attempts INT NOT NULL DEFAULT 1;
DELETE FROM lockout WHERE attempts < 1;

ALTER TABLE data_bases MODIFY name CHAR(63) CHARACTER SET ascii COLLATE ascii_bin NOT NULL;

DELETE FROM db_users WHERE account_id IS NULL;
ALTER TABLE db_users MODIFY account_id INT UNSIGNED NOT NULL;

UPDATE sys_users SET quota = 0 WHERE quota IS NULL;
ALTER TABLE sys_users MODIFY quota BIGINT UNSIGNED NOT NULL DEFAULT 0;

ALTER TABLE Permissions MODIFY permission CHAR(245) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL;

ALTER TABLE Parameters MODIFY parameter CHAR(245) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL;

ALTER TABLE Limits MODIFY limit_name CHAR(245) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL;

ALTER TABLE TmplData MODIFY element CHAR(245) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL;

ALTER TABLE slave_params MODIFY parameter VARCHAR(245) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL;

ALTER TABLE ListsParams MODIFY list_name VARCHAR(225) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL DEFAULT '';

DELETE FROM misc WHERE val IS NULL;
ALTER TABLE misc MODIFY val VARCHAR(255) CHARACTER SET binary NOT NULL;

ALTER TABLE custom_buttons MODIFY id INT UNSIGNED AUTO_INCREMENT;
UPDATE custom_buttons SET text = '' WHERE TEXT IS NULL;
ALTER TABLE custom_buttons MODIFY text VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL;
UPDATE custom_buttons SET options = 0 WHERE options IS NULL;
ALTER TABLE custom_buttons MODIFY options INT UNSIGNED NOT NULL DEFAULT 0;

DELETE FROM upgrade_history WHERE upgrade_date IS NULL;
ALTER TABLE upgrade_history MODIFY upgrade_date DATETIME NOT NULL;
UPDATE upgrade_history SET version_info = '' WHERE version_info IS NULL;
ALTER TABLE upgrade_history MODIFY version_info VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin NOT NULL;
UPDATE upgrade_history SET db_version = '' WHERE db_version IS NULL;
ALTER TABLE upgrade_history MODIFY db_version VARCHAR(8) CHARACTER SET ascii COLLATE ascii_bin NOT NULL;

DELETE FROM log_actions WHERE date IS NULL;
ALTER TABLE log_actions MODIFY date DATETIME NOT NULL;
ALTER TABLE log_components MODIFY component VARCHAR(245)  NOT NULL;
DELETE FROM log_components WHERE component = 'Unknown';

UPDATE event_handlers SET priority = 0 WHERE priority IS NULL;
ALTER TABLE event_handlers MODIFY priority INT UNSIGNED NOT NULL DEFAULT 0;
UPDATE event_handlers SET user = 'nobody' WHERE user IS NULL;
ALTER TABLE event_handlers MODIFY user VARCHAR(255) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL DEFAULT 'nobody';
UPDATE event_handlers SET command = '' WHERE command IS NULL;
ALTER TABLE event_handlers MODIFY command TEXT NOT NULL;

ALTER TABLE report MODIFY id INT UNSIGNED NOT NULL AUTO_INCREMENT;
ALTER TABLE report MODIFY type INT UNSIGNED NOT NULL;
ALTER TABLE report MODIFY user_id INT UNSIGNED NOT NULL DEFAULT 0;
ALTER TABLE report MODIFY user_type INT UNSIGNED NOT NULL;
UPDATE report SET name = '' WHERE name IS NULL;
ALTER TABLE report MODIFY name VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL;

ALTER TABLE report_section MODIFY id INT UNSIGNED NOT NULL AUTO_INCREMENT;
ALTER TABLE report_section MODIFY report_id INT UNSIGNED NOT NULL;

ALTER TABLE report_auto MODIFY id INT UNSIGNED NOT NULL AUTO_INCREMENT;
ALTER TABLE report_auto MODIFY report_id INT UNSIGNED NOT NULL;
ALTER TABLE report_auto MODIFY auto INT UNSIGNED NOT NULL DEFAULT 0;
ALTER TABLE report_auto MODIFY recipient INT UNSIGNED NOT NULL DEFAULT 0;
ALTER TABLE report_auto MODIFY client INT UNSIGNED;
ALTER TABLE report_auto MODIFY domain INT UNSIGNED;

ALTER TABLE SiteApps MODIFY params_id INT UNSIGNED;

ALTER TABLE tts_reporters MODIFY ref_reporter_id INT UNSIGNED;

DELETE FROM key_history_params WHERE val IS NULL;
ALTER TABLE key_history_params MODIFY val VARCHAR(255) CHARACTER SET binary NOT NULL;

DELETE FROM misc WHERE param="lines_per_page";
DELETE FROM mn_param WHERE param="lines_per_page";
DELETE FROM dom_param WHERE param="lines_per_page";
DELETE FROM cl_param WHERE param="lines_per_page";
DELETE FROM TmplData WHERE element="lines_per_page";

replace into Notes (id, text) values (8, "A new ticket with ID <ticket_id> has been submitted to the Help Desk by <reporter> from <server>.\nPlease retain the ticket identification number for future references. Below is a copy of the ticket.\n-----\n<ticket_comment>\n-----");
replace into Notes (id, text) values (9, "Ticket <ticket_id> has been commented by <reporter> from <server>.\nBelow is the comment.\n-----\n<ticket_comment>\n-----");
replace into Notes (id, text) values (10,"Ticket <ticket_id> has been closed by <reporter> from <server>.\nBelow is the comment.\n-----\n<ticket_comment>\n-----");
replace into Notes (id, text) values (11,"Ticket <ticket_id> has been reopened by <reporter> from <server>.\nBelow is the comment.\n-----\n<ticket_comment>\n-----");

delete from Skins;
replace into Skins (id, name, place) values (1, 'WinXP Blue','winxp.blue');
replace into Skins (id, name, place) values (2, 'WinXP Olivegreen','winxp.olivegreen');
replace into Skins (id, name, place) values (3, 'WinXP Silver','winxp.silver');
replace into Skins (id, name, place) values (4, 'Plesk Silver','plesk.silver');
replace into Skins (id, name, place) values (5, 'Plesk Classic Silver','plesk.classic.silver');
replace into Skins (id, name, place) values (6, 'WinXP Reloaded','winxp.new');
replace into Skins (id, name, place) values (7, 'Aqua','aqua');
replace into Skins (id, name, place) values (8, 'WinXP Reloaded Compact','winxp.new.compact');
replace into Skins (id, name, place) values (9, 'Aqua Compact','aqua.compact');
replace into Skins (id, name, place) values (10, 'Longhorn Aero','longhorn.aero');
replace into Skins (id, name, place) values (11, 'Longhorn Black','longhorn.black');
replace into Skins (id, name, place) values (12, 'Longhorn Yellow','longhorn.yellow');
replace into Skins (id, name, place) values (13, 'Plesk Sea','plesk.sea');
replace into Skins (id, name, place) values (14, 'Plesk Violet','plesk.violet');
replace into Skins (id, name, place) values (15, 'Plesk Nature','plesk.nature');
replace into Skins (id, name, place) values (16, 'Plesk Blue','plesk.blue');

replace into actions(name, descr, enabled) values ('database_server_create', 'Create database server', 'true');
replace into actions(name, descr, enabled) values ('database_server_update', 'Update database server', 'true');
replace into actions(name, descr, enabled) values ('database_server_delete', 'Delere database server', 'true');
replace into actions(name, descr, enabled) values ('domain_alias_create', 'Create domain alias', 'true');
replace into actions(name, descr, enabled) values ('domain_alias_update', 'Update domain dlias', 'true');
replace into actions(name, descr, enabled) values ('domain_alias_delete', 'Delete domain alias', 'true');
UPDATE misc SET val = 8 WHERE param IN ('def_skin_id', 'admin_skin_id') AND val > 16;
UPDATE cl_param SET val = 8 WHERE param = 'skin_id' AND val > 16;
UPDATE dom_param SET val = 8 WHERE param = 'skin_id' AND val > 16;
UPDATE mn_param SET val = 8 WHERE param = 'skin_id' AND val > 16;

UPDATE tts_queues SET enabled = 'false' WHERE enabled != 'true';
UPDATE tts_cats SET enabled = 'false' WHERE enabled != 'true';
UPDATE tts_priorities SET enabled = 'false' WHERE enabled != 'true';

DELETE FROM actions WHERE name='siteapp_unchain';

UPDATE actions set descr = 'Domain Administrator Update' WHERE name='dl_user_update';

ALTER TABLE accounts MODIFY type VARCHAR(32) NOT NULL DEFAULT 'plain';
ALTER TABLE accounts MODIFY password TEXT CHARACTER SET ascii COLLATE ascii_bin;

-- PMM table operations are performed in psa-migration-manager
DROP TABLE IF EXISTS PMM;
